Package: ei
Version: 1.3-3
Date: 2016-11-04
Title: Ecological Inference
Author: Gary King <king@harvard.edu>, Molly Roberts <molly.e.roberts@gmail.com>
Maintainer: James Honaker <zelig.zee@gmail.com>
Depends: R (>= 2.5.0), eiPack
Imports: mvtnorm, msm, tmvtnorm, ellipse, plotrix, MASS, ucminf,
        cubature, mnormt, foreach, sp
Suggests: rgl
Description: Software accompanying Gary King's book: A Solution to the Ecological Inference Problem. (1997). Princeton University Press. ISBN 978-0691012407. 
License: GPL (>= 2)
URL: http://gking.harvard.edu/eiR,
        http://gking.harvard.edu/eicamera/kinroot.html
NeedsCompilation: no
Packaged: 2016-11-04 22:19:49 UTC; tercer
Repository: CRAN
Date/Publication: 2016-11-05 00:49:10
Built: R 4.6.0; ; 2026-04-29 02:13:18 UTC; unix

Package: ei
Version: 1.3-3
Date: 2016-11-04
Title: Ecological Inference
Author: Gary King <king@harvard.edu>, Molly Roberts <molly.e.roberts@gmail.com>
Maintainer: James Honaker <zelig.zee@gmail.com>
Depends: R (>= 2.5.0), eiPack
Imports: mvtnorm, msm, tmvtnorm, ellipse, plotrix, MASS, ucminf,
        cubature, mnormt, foreach, sp
Suggests: rgl
Description: Software accompanying Gary King's book: A Solution to the Ecological Inference Problem. (1997). Princeton University Press. ISBN 978-0691012407. 
License: GPL (>= 2)
URL: http://gking.harvard.edu/eiR,
        http://gking.harvard.edu/eicamera/kinroot.html
NeedsCompilation: no
Packaged: 2016-11-04 22:19:49 UTC; tercer
Repository: CRAN
Date/Publication: 2016-11-05 00:49:10
Built: R 4.6.0; ; 2026-04-26 12:53:18 UTC; unix

 export(ei,
       ei.sim,
       eiread,
       bounds1,
       tomogRxC,
       tomogRxC3d)

import(mvtnorm)
import(msm)
import(tmvtnorm)
import(ellipse)
import(MASS)
import(ucminf)
import(cubature)
import(mnormt)
import(foreach)
import(eiPack)
import(sp)

importFrom("plotrix","draw.circle")

importFrom("grDevices", "chull", "hcl", "heat.colors", "rainbow", "rgb")
importFrom("graphics", "abline", "image", "legend", "lines", "mtext", "par", "points", "polygon", "text")
importFrom("stats", "coef", "cor", "cov", "density", "dnorm", "lm", "na.omit", "pnorm", "quantile", "runif", "sd", "terms.formula", "var", "weighted.mean")
importFrom("utils", "head")

S3method(plot, ei)
S3method(summary, ei)

#@x is an ei.object

summary.ei <- function(object,...){
  if("psi" %in% names(object)){
  ei1 <- object
  #Calculate maximum likelihood results in the scale of estimation
  numb <- ei1$numb
  covs <- as.logical(ifelse(diag(ei1$hessian)==0,0,1))
  sdphi <- sqrt(diag((solve(ei1$hessian[covs,covs]))))
  zbmiss <- ifelse(covs[6:(5+numb)]==FALSE,TRUE,FALSE)
  zwmiss <- ifelse(covs[(6+numb):length(covs)]==FALSE, TRUE, FALSE)
  names <- c("Bb0", "Bw0", "sigB", "sigW", "rho")
  if(zbmiss==FALSE&zwmiss==FALSE){
    sdphi <- c(sdphi[1:5], 0,0)
    names <- c(names, "Zb", "Zw")
  }
  if(zbmiss==TRUE&zwmiss==FALSE){
    sdphi <- c(sdphi[1:5], 0, sdphi[(5+numb):sum(covs)])
    numw <- length(ei1$phi) - (5+numb)
    wname <- NULL
    for (i in 1:numw){
      wname[i] = paste("Zw",(i-1), sep="")
    }
    names <- c(names, "Zb0", wname)
  }
  if(zbmiss==FALSE&zwmiss==TRUE){
    sdphi <- c(sdphi, 0)
    bname <- NULL
    for (i in 1:numb){
      bname[i] = paste("Zb",(i-1), sep="")
    }
    names <- c(names, bname, "Zw0")
  }
  if(zbmiss==TRUE&zwmiss==TRUE){
    sdphi <- c(sdphi, 0, 0)
    names <- c(names, "Zb0", "Zw0")
  }
  mle <- rbind(ei1$phi, sdphi)
  colnames(mle) <- names
  rownames(mle) <- c("","")
 #Calculate untruncated psis
  n <- length(ei1$betab)
  BB <- mean(ei1$psi[,1:n])
  BW <- mean(ei1$psi[,(n+1):(2*n)])
  SB <- mean(ei1$psi[,((2*n)+1)])
  SW <- mean(ei1$psi[,((2*n)+2)])
  RHO <- mean(ei1$psi[,((2*n)+3)])
  psiu <- t(matrix(c(BB,BW,SB,SW,RHO)))
  colnames(psiu) <- c("BB","BW", "SB", "SW", "RHO")
  rownames(psiu) <- c("")
#Calculate truncated psis
  BB <- mean(na.omit(ei1$betabs))
  BW <- mean(na.omit(ei1$betaws))
  SB <- sd(as.vector(na.omit(ei1$betabs)))
  SW <- sd(as.vector(na.omit(ei1$betaws)))
  mat <- na.omit(cbind(as.vector(ei1$betabs), as.vector(ei1$betaws)))
  RHO <- cor(mat[,1],mat[,2])
  psit <- t(matrix(c(BB,BW,SB,SW,RHO)))
  colnames(psit) <- c("BB","BW", "SB", "SW", "RHO")
  rownames(psit) <- c("")

#Aggregate Bounds
  ab <- matrix(.abounds(ei1), nrow=2)
  rownames(ab) <- c("lower", "upper")
  colnames(ab) <- c("betab", "betaw")

#Estimates of Aggregate Quantities of Interest
  magg <- matrix(.maggs(ei1), nrow=2)
  rownames(magg) <- c("Bb", "Bw")
  colnames(magg) <- c("mean", "sd")
  output <- list(ei1$erho, ei1$esigma, ei1$ebeta, n, ei1$resamp,
                 mle, psiu, psit, ab, magg, ei1$precision)
  names(output) <- c("Erho", "Esigma", "Ebeta", "N", "Resamp",
        "Maximum likelihood results in scale of estimation (and se's)",
        "Untruncated psi's", "Truncated psi's (ultimate scale)",
        "Aggregate Bounds", "Estimates of Aggregate Quantities of Interest", "precision")
  class(output) <- "summary"
  return(output)
}
  if (!("psi"%in% names(object))){

ei1 <- object
  #Calculate maximum likelihood results in the scale of estimation
  n <- length(ei1$x)
  numb <- ei1$numb
  covs <- as.logical(ifelse(diag(ei1$hessian)==0,0,1))
  sdphi <- sqrt(diag((solve(ei1$hessian[covs,covs]))))
  zbmiss <- ifelse(covs[6:(5+numb)]==FALSE,TRUE,FALSE)
  zwmiss <- ifelse(covs[(6+numb):length(covs)]==FALSE, TRUE, FALSE)
  names <- c("Bb0", "Bw0", "sigB", "sigW", "rho")
  if(zbmiss==FALSE&zwmiss==FALSE){
    sdphi <- c(sdphi[1:5], 0,0)
    names <- c(names, "Zb", "Zw")
  }
  if(zbmiss==TRUE&zwmiss==FALSE){
    sdphi <- c(sdphi[1:5], 0, sdphi[(5+numb):sum(covs)])
    numw <- length(ei1$phi) - (5+numb)
    wname <- NULL
    for (i in 1:numw){
      wname[i] = paste("Zw",(i-1), sep="")
    }
    names <- c(names, "Zb0", wname)
  }
  if(zbmiss==FALSE&zwmiss==TRUE){
    sdphi <- c(sdphi, 0)
    bname <- NULL
    for (i in 1:numb){
      bname[i] = paste("Zb",(i-1), sep="")
    }
    names <- c(names, bname, "Zw0")
  }
  if(zbmiss==TRUE&zwmiss==TRUE){
    sdphi <- c(sdphi, 0, 0)
    names <- c(names, "Zb0", "Zw0")
  }
  mle <- rbind(ei1$phi, sdphi)
  colnames(mle) <- names
  rownames(mle) <- c("","")
#Aggregate Bounds
  ab <- matrix(.abounds(ei1), nrow=2)
  rownames(ab) <- c("lower", "upper")
  colnames(ab) <- c("betab", "betaw")
 output <- list(ei1$erho, ei1$esigma, ei1$ebeta, n,
                 mle,ab, ei1$precision)
  names(output) <- c("Erho", "Esigma", "Ebeta", "N",
        "Maximum likelihood results in scale of estimation (and se's)","Aggregate Bounds","precision")
  class(output) <- "summary"
  return(output)

  }
}

#@numb - number of covariates for bb
#@covs - number of total parameters to estimate
#@all the rest in documentation

#Samp implements importance sampling

.samp <- function(t,x,n, Zb, Zw, par, varcv, nsims, keep, numb, covs,
                  erho, esigma, ebeta, ealphab, ealphaw, Rfun){
  import1 <- NULL

  varcv2 <- solve(varcv)/4

  draw <- rmvnorm(nsims, par[covs], varcv2)
  varcv3 <- solve(varcv2)
  phiv <- dmvnorm(draw, par[covs], varcv2, log=T)
  zbmiss <- ifelse(covs[6] == FALSE,TRUE,FALSE)
  zwmiss <- ifelse(covs[(6+numb)] == FALSE, TRUE, FALSE)
  if(zbmiss == TRUE & zwmiss == FALSE){
    draw <- cbind(draw[,1:5], rep(1,nsims), draw[,(5+numb):sum(covs)])
  }
  if(zbmiss == FALSE & zwmiss == TRUE){
    draw <- cbind(draw, rep(1,nsims))
  }
  if(zbmiss == TRUE & zwmiss == TRUE){
    draw <- cbind(draw, rep(1,nsims), rep(1,nsims))
  }
  #for(i in 1:nsims){
      #import1[i] <- -like(as.vector(draw[i,]),
      #t, x, n, Zb, Zw, numb=numb, erho, esigma, ebeta,
      #ealphab, ealphaw, Rfun) - phiv[i]
	#}
  
  #Calculates importance ratio
  import1 <- apply(as.matrix(1:nsims),1,function(i)
                  -like(as.vector(draw[i,]),t, x, n, Zb, Zw,
                  numb=numb, erho, esigma, ebeta, ealphab, ealphaw,Rfun)
                   - phiv[i])

  ok <- !is.nan(import1)
  lnir <- import1-max(import1[ok])
  ir <- NA
  ir[ok] <- exp(lnir[ok])
  #print(mean(is.finite(ir)))
  tst <- ifelse(is.finite(ir), ir>runif(1,0,1), FALSE)
  #print(sum(tst))
  keep <- rbind(keep, draw[tst,])
  return(keep)
}

                                   #@sub -- indeces to create R for
#@Rfun - R function to use
#@bb,bw,sb,sw,rho -- current MLE estimates
#numb numw -- number of covaraites for bb and bw

.createR <- function(sub, Rfun, bb, bw, sb,sw, rho, x, numb, numw){
  out <- NULL 
  lower = cbind(-bb[sub]/sb, -bw[sub]/sw)
  upper = cbind(-bb[sub]/sb+1/sb, -bw[sub]/sw+1/sw)
  mean=c(0,0)
  corr=matrix(c(1,rho,rho,1),nrow=2)

  if (Rfun==1){
    out <- NULL
    makeR <- function (i){
      qi <- pmvnorm(lower=lower[i,], upper=upper[i,], mean=mean,
                    corr=corr)
    }
    out <- foreach(i = 1:length(x[sub]), .combine="c") %dopar% makeR(i)
    #out <- apply(as.matrix(1:length(x[sub])), 1, makeR)
    out <- ifelse(out<0|out==0, 1*10^-322,out)
    out <- log(out)
    #if(sum(is.na(out))>0|sum((out==Inf))>0) print("R not real")
    out <- ifelse(is.na(out)|abs(out==Inf), 999, out)
    return(out)
  }
  if (Rfun==2){
    makeR <- function(i){
      qi <- sadmvn(lower=lower[i,], upper=upper[i,], mean=mean,
                   varcov=corr)
    }
    #out <- foreach(i = 1:length(x[sub]), .combine="c") %dopar% makeR(i)
    out <- apply(as.matrix(1:length(x[sub])), 1, makeR)
    out <- ifelse(out<0|out==0, 1*10^-322,out)
    out <- log(out)
    #if(sum(is.na(out))>0|sum((out==Inf))>0) print("R not real")
    out <- ifelse(is.na(out)|abs(out==Inf), 999, out)
        #return(out)
        # for(i in 1:length(x[sub])){
        # qi <- sadmvn(lower=lower[i,], upper=upper[i,], mean=mean, varcov=corr)
        #qi <- ifelse(qi<0|qi==0, 1*10^-322,qi)
        #out[i] <- log(qi)
        #if(is.na(out[i])|abs(out[i]==Inf)) print("R not real")
        #out[i] <- ifelse(is.na(out[i])|abs(out[i]==Inf), 999, out[i])
    #}
    return(out)
  }
  if (Rfun==3){
    fun <- function(x) dmvnorm(x,mean,corr)
    for (i in 1:length(x[sub])){
      qi <- adaptIntegrate(fun, lowerLimit=lower[i,],
                           upperLimit=upper[i,])$integral
      out[i] <- log(qi)
      #if(is.na(out[i])|abs(out[i]==Inf)) print("R not real")
      out[i] <- ifelse(is.na(out[i])|abs(out[i]==Inf), 999, out[i])
    }
    return(out)
  }
  ## THIS VERSION RELIES ON mvnprd FROM PACKAGE mvtnormpcs WHICH IS NOW ARCHIVED
    if (Rfun==4){
        print("Option Rfun==4 is no longer possible")
        stop()
  ##  for(i in 1:length(x[sub])){
  ##    qi <- mvnprd(A=upper[i,], B=lower[i,],
  ##                 BPD=c(rho,rho),INF=rep(2,2))$PROB
  ##    qi <- ifelse(qi<0|qi==0, 1*10^-322,qi)
  ##    out[i] <- log(qi)
      #if(is.na(out[i])|abs(out[i]==Inf)) print("R not real")
  ##    out[i] <- ifelse(is.na(out[i])|abs(out[i]==Inf), 999, out[i])
  ##  }
  ##  return(out)
  }

if (Rfun==5){
  lower = lower[1,]
  upper = upper[1,]
  #qi <- pmvnorm(lower=lower, upper=upper, mean=mean, corr=corr)
  qi <- sadmvn(lower=lower, upper=upper, mean=mean, varcov=corr)
  qi <- ifelse(qi<1*10^-14, 1*10^-14,qi)
  qi <- log(qi)
  #if(is.na(qi)|abs(qi)==Inf) print ("R not real")
  qi <- ifelse((is.na(qi)|abs(qi)==Inf),999,qi)
  out <- rep(qi,length(x[sub]))
  return(out)
}
}



#@par - solution to likelihood maximization
#@numb - number of covariates for bb
#@covs - number of total parameters to estimate
#@all the rest in documentation

#Samp implements importance sampling

.samp <- function(t,x,n, Zb, Zw, par, varcv, nsims, keep, numb, covs,
                  erho, esigma, ebeta, ealphab, ealphaw, Rfun){
  import1 <- NULL
  varcv2 <- solve(varcv)/4
  draw <- rmvnorm(nsims, par[covs], varcv2)
  varcv3 <- solve(varcv2)
  phiv <- dmvnorm(draw, par[covs], varcv2, log=T)
  #print("samp")
  zbmiss <- ifelse(covs[6] == FALSE,TRUE,FALSE)
  zwmiss <- ifelse(covs[(6+numb)] == FALSE, TRUE, FALSE)
  if(zbmiss == TRUE & zwmiss == FALSE){
    draw <- cbind(draw[,1:5], rep(1,nsims), draw[,(5+numb):sum(covs)])
  }
  if(zbmiss == FALSE & zwmiss == TRUE){
    draw <- cbind(draw, rep(1,nsims))
  }
  if(zbmiss == TRUE & zwmiss == TRUE){
    draw <- cbind(draw, rep(1,nsims), rep(1,nsims))
  }
  #for(i in 1:nsims){
      #import1[i] <- -like(as.vector(draw[i,]),
      #t, x, n, Zb, Zw, numb=numb, erho, esigma, ebeta,
      #ealphab, ealphaw, Rfun) - phiv[i]
	#}
  
  #Calculates importance ratio
  import1 <- apply(as.matrix(1:nsims),1,function(i)
                  -like(as.vector(draw[i,]),t, x, n, Zb, Zw,
                  numb=numb, erho, esigma, ebeta, ealphab, ealphaw,Rfun)
                   - phiv[i])
  ok <- !is.nan(import1)
  lnir <- import1-max(import1[ok])
  ir <- NA
  ir[ok] <- exp(lnir[ok])
  #print(mean(is.finite(ir)))
  tst <- ifelse(is.finite(ir), ir>runif(1,0,1), FALSE)
  #print(sum(tst))
  keep <- rbind(keep, draw[tst,])
  return(keep)
}

#@All arguments are values on the untruncated scale
#This function reparameterizes to the truncated scale

.repar <- function(Bb0, Bw0, sb0, sw0, rho0, Bb0v, Bw0v, Zb, Zw){
  sb=exp(sb0)
  sw=exp(sw0)
  bb=Bb0*(.25+sb^2) + .5 +
    as.matrix(apply(Zb,2, function(x) x - mean(x)))%*%as.matrix(Bb0v)
  bw=Bw0*(.25+sw^2) + .5 +
    as.matrix(apply(Zw,2, function(x) x - mean(x)))%*%as.matrix(Bw0v)
  rho=(exp(2*rho0)-1)/(exp(2*rho0) +1)
  return(c(t(bb), t(bw), sb, sw, rho))
}

#Functions for plotting

#Tomography plot

.tomog <- function(ei.object,title="Tomography Plot with the Data",lci=T){
  x <- ei.object$x
  t <- ei.object$t
  n <- ei.object$n
  #Take out the bounds
  bounds <- bounds1(x,t,n)
  bbounds <- cbind(bounds[,1], bounds[,2])
  wbounds <- cbind(bounds[,4], bounds[,3])
  #Number of precincts n
  n <- dim(bounds)[1]
  #Figure out length of line for LOCOI
  length<-NA
  for(i in 1:n){
    length[i]<-sqrt(abs(bbounds[i,1]-bbounds[i,2])^2 + abs(wbounds[i,1]-wbounds[i,2])^2)
  }
  scale<-((length-min(length))/(max(length)-min(length)))*100
  #Plot
  plot(c(100,200), xlim=c(0,1), ylim=c(0,1),
       col="white", ylab="betaW", xlab="betaB", xaxs="i",
       yaxs="i", main=title)
  if(lci==T){for(i in 1:n){lines(bbounds[i,], wbounds[i,], col=hcl(h=30,c=100,l=scale[i],alpha=1))}}
  else{for(i in 1:n){lines(bbounds[i,], wbounds[i,], col="yellow")}}
}

.tomogd <- function(x,t,n,title,lci=T){
  bounds <- bounds1(x,t,n)
  bbounds <- cbind(bounds[,1], bounds[,2])
  wbounds <- cbind(bounds[,4], bounds[,3])
  n <- dim(bounds)[1]
  #Figure out length of line for LOCOI
  length<-NA
  for(i in 1:n){
      length[i]<-sqrt(abs(bbounds[i,1]-bbounds[i,2])^2 + abs(wbounds[i,1]-wbounds[i,2])^2)
  }
  scale<-((length-min(length))/(max(length)-min(length)))*100
  plot(c(100,200), xlim=c(0,1), ylim=c(0,1),
       col="white", ylab="betaW", xlab="betaB", xaxs="i",
       yaxs="i", main=title)
  if(lci==T){for(i in 1:n){lines(bbounds[i,], wbounds[i,], col=hcl(h=30,c=100,l=scale[i],alpha=1))}}
  else{for(i in 1:n){lines(bbounds[i,], wbounds[i,], col="yellow")}}
}

#Tomography plot with ML contours
.tomogl <- function(ei.object, lci=TRUE){
  x <- ei.object$x
  t <- ei.object$t
  n <- ei.object$n
  Zb <- ei.object$Zb
  Zw <- ei.object$Zw
  phi <- ei.object$phi
  .tomogd(x,t,n, "Tomography Plot with ML Contours", lci=lci)
  numb <- dim(Zb)[2]
  numw <- dim(Zw)[2]
  Bb0 <- phi[1]
  Bw0 <- phi[2]
  sb0 <- phi[3]
  sw0 <- phi[4]
  rho0 <- phi[5]
  Bb0v <- phi[6:(5+numb)]
  Bw0v <- phi[(6+numb):length(phi)]
  vars <- .repar(Bb0,Bw0,sb0,sw0,rho0, Bb0v, Bw0v, Zb, Zw)
  bb <- vars[1:length(x)]
  bw <- vars[(length(x)+1):(2*length(x))]
  sb <- vars[2*length(x)+1]
  sw <- vars[2*length(x)+2]
  rho <- vars[2*length(x)+3]
  .tomog3 <- function(bb,bw,sb,sw,rho){
    lines(ellipse(matrix(c(1,rho,rho,1),nrow=2), scale=c(sb,sw),
        centre=c(mean(bb),mean(bw)),level=.914), col="blue",lwd=4)
    lines(ellipse(matrix(c(1,rho,rho,1),nrow=2), scale=c(sb,sw),
        centre=c(mean(bb),mean(bw)),level=.35), col="red",lwd=4)
    points(mean(bb),mean(bw),col="pink",  pch=15)
  }
  
#tomog4 <- function(bb,bw,sb,sw,rho){
#	lines(ellipse(matrix(c(1,rho,rho,1),nrow=2), #scale=c(sb,sw),centre=c(mean(bb),mean(bw)),level=.914), #col="blue",lwd=1, lty=3)
#	lines(ellipse(matrix(c(1,rho,rho,1),nrow=2), #scale=c(sb,sw),centre=c(mean(bb),mean(bw)),level=.35), #col="red",lwd=1, lty=3)
#	}
#or
#for (i in 1:length(x)){
	#points(mean(bb[i]), mean(bw[i]), col="red", #pch=19, cex=.1)
#	tomog3(bb[i], bw[i], sb, sw, rho)
	#}
  .tomog3(bb,bw,sb,sw,rho)
}

.tomog3 <- function(bb,bw,sb,sw,rho){
  lines(ellipse(matrix(c(1,rho,rho,1),nrow=2),
        scale=c(sb,sw),centre=c(mean(bb),mean(bw)),level=.914),
        col="blue",lwd=4)
  lines(ellipse(matrix(c(1,rho,rho,1),nrow=2), scale=c(sb,sw),
        centre=c(mean(bb),mean(bw)),level=.35), col="red",lwd=4)
  points(mean(bb),mean(bw),col="pink",  pch=15)
}


#Tomography plot with 80% CIs
.tomog80CI <- function(ei.object){
  if(!("betabs"%in% names(ei.object))){
   message("Error: This plot function requires an ei.sim object.")
  }
  if("betabs"%in% names(ei.object)){
  #Only consider precincts that are heterogeneous
  ok <- !is.na(ei.object$betab)&!is.na(ei.object$betaw)
  x <- ei.object$x[ok]
  t <- ei.object$t[ok]
  n <- ei.object$n[ok]
  betabs <- ei.object$betabs[ok,]
  betaws <- ei.object$betaws[ok,]
  .tomogd(x,t,n,"Tomography Plot with 80% CIs",lci=F)
  #Create confidence intervales
  betabcd <- apply(betabs,1,function(x) quantile(x, probs=c(.1,.9)))
  betawcd <- apply(betaws,1,function (x) quantile(x,probs=c(.1,.9)))
  n <- dim(betabcd)[2]
  for(i in 1:n){
    lines(betabcd[,i], sort(betawcd[,i],decreasing=T), col="red",
          lwd=3)
  }
  }
}

#Tomography plot with 95% CIs
.tomog95CI <- function(ei.object){
  if(!("betabs"%in% names(ei.object))){
   message("Error: This plot function requires an ei.sim object.")
  }
  if("betabs"%in% names(ei.object)){
  ok <- !is.na(ei.object$betab)&!is.na(ei.object$betaw)
  x <- ei.object$x[ok]
  t <- ei.object$t[ok]
  n <- ei.object$n[ok]
  betabs <- ei.object$betabs[ok,]
  betaws <- ei.object$betaws[ok,]
  .tomogd(x,t,n,"Tomography Plot with 95% CIs",lci=F)
  betabcd <- apply(betabs,1,function(x) quantile(x,
                                                 probs=c(.025,.975)))
  betawcd <- apply(betaws,1,function (x)
                   quantile(x,probs=c(.025,.975)))
  n <- dim(betabcd)[2]
  for(i in 1:n){
    lines(betabcd[,i], sort(betawcd[,i], decreasing=T), col="red",
          lwd=3)
  }
}
}

#TomogE -- Tomography plot with mean posterior betabs and betaws
.tomogE <- function(ei.object){
  if(!("betabs"%in% names(ei.object))){
   message("Error: This plot function requires an ei.sim object.")
  }
  if("betabs"%in% names(ei.object)){
  ok <- !is.na(ei.object$betab)&!is.na(ei.object$betaw)
  x <- ei.object$x[ok]
  t <- ei.object$t[ok]
  n <- ei.object$n[ok]
  betabs <- ei.object$betabs[ok,]
  betaws <- ei.object$betaws[ok,]
  .tomogd(x,t,n,"Tomography Plot with Mean Posterior Betabs and
Betaws")
  betabm <- apply(betabs,1,mean)
  betawm <- apply(betaws,1,mean)
  points(betabm, betawm, col="red", pch=19)
}
}

#TomogP -- Tomography plot with contours based on mean posterior psi
#tomogd(data$x,data$t, data$n, "Tomography with simulated contours based on mean posterior")
#vars <- apply(psi,2,mean)
#bb <- vars[1]
#bw <- vars[2]
#sb <- vars[3]
#sw <- vars[4]
#rho <- vars[5]
#tomog(bb,bw,sb,sw,rho)
#dev.off()


.tomogP2 <- function(ei.object){
  if(!("betabs"%in% names(ei.object))){
   message("Error: This plot function requires an ei.sim object.")
  }
  if("betabs"%in% names(ei.object)){
  x <- ei.object$x
  t <- ei.object$t
  n <- ei.object$n
  psi <- ei.object$psi
  .tomogd(x,t,n,"Tomography Plot with Contours Based on Posterior")
  bbp <- psi[,1:length(x)]
  bwp <- psi[,(length(x)+1):(2*length(x))]
  sbp <- psi[,2*length(x)+1]
  swp <- psi[,2*length(x)+2]
  rhop <- psi[,2*length(x)+3]
  points(mean(bbp), mean(bwp), col="red", pch=19)
  .tomog3(bbp,bwp,mean(sbp),mean(swp),mean(rhop))
#or
#points(mean(bbp), mean(bwp), col="red", pch=19)
#for (i in 1:length(x)){
#	points(mean(bbp[,i]), mean(bwp[,i]), col="red", #pch=19, cex=.1)
#	tomog4(bbp[,i], bwp[,i], mean(sbp), mean(swp), mean #(rhop))
#	}
}
}


#Density plots
#tomogd(data$x,data$t,data$n, "Tomography with contours #from posterior")
#bivn.kde <- kde2d(psi[,1], psi[,2], n=100)
#contour(bivn.kde, add=T, nlevels=7, drawlabels=F)
#points(mean(psi[,1]), mean(psi[,2]), col="red", pch=15)

#Density of betab
.betabd <- function(ei.object){
  if(!("betabs"%in% names(ei.object))){
   message("Error: This plot function requires an ei.sim object.")
  }
  if("betabs"%in% names(ei.object)){
  ok <- !is.na(ei.object$betab)
  betabs <- ei.object$betabs[ok,]
  betabm <- apply(betabs,1,mean)
  plot(density(betabm), xlim=c(0,1),  col="green", xlab="betaB",
       ylab="density across precincts, f(betaB)", main="Density of
betaB")
  vb <- as.vector(betabm)
  for (i in 1:length(vb)){
    lines(c(vb[i], vb[i]), c(0,.25))
  }
}
}

#Density of betaw
.betawd <- function(ei.object){
  if(!("betabs"%in% names(ei.object))){
   message("Error: This plot function requires an ei.sim object.")
  }
  if("betabs"%in% names(ei.object)){
  ok <- !is.na(ei.object$betaw)
  betaws <- ei.object$betaws[ok,]
  betawm <- apply(betaws,1,mean)
  plot(density(betawm), xlim=c(0,1), col="green", xlab="betaW",
       ylab="density across precincts, f(betaW)", main="Density of
betaW")
  vw <- as.vector(betawm)
  for (i in 1:length(vw)){
    lines(c(vw[i], vw[i]), c(0,.25))
  }
}
}


#XT plot
.xt <- function(ei.object){
  x <- ei.object$x
  t <- ei.object$t
  plot(x, t, xlim=c(0,1), ylim=c(0,1),xaxs="i",yaxs="i", main="X and T
Scatterplot", ylab="T", xlab="X", pch=20)
}

#XTc plot

.xtc <- function(ei.object){
  x <- ei.object$x
  t <- ei.object$t
  n <- ei.object$n
  circ <- .04
  plot(x, t, xlim=c(0,1), ylim=c(0,1),xaxs="i",yaxs="i", main="X and T
Scatterplot with Population Density Circles", ylab="T", xlab="X",
       pch=20)
  minn <- min(n)
  maxn <- max(n)
  for (i in 1:length(x)){
    radius = (n[i]-minn+1)/(1+maxn-minn)
    draw.circle(x[i], t[i], radius*circ)
  }
}
#xtc(data$x,data$t,data$n,.04, "X and T Scatterplot")


#XTfit plot

.xtfit <- function(ei.object){
  if(!("betabs"%in% names(ei.object))){
   message("Error: This plot function requires an ei.sim object.")
  }
  if("betabs"%in% names(ei.object)){
  ok <- !is.na(ei.object$betab)&!is.na(ei.object$betaw)
  x <- ei.object$x[ok]
  t <- ei.object$t[ok]
  n <- ei.object$n[ok]
  betabs <- ei.object$betabs[ok,]
  betaws <- ei.object$betaws[ok,]
  low <- .1
  up <- .9
  circ <- .04
  plot(x, t, xlim=c(0,1), ylim=c(0,1),xaxs="i",yaxs="i", main="X and T
Scatterplot with E(T|X) and 80% CIs", ylab="T", xlab="X", pch=20)
  minn <- min(n)
  maxn <- max(n)
  for (i in 1:length(x)){
    radius = (n[i]-minn+1)/(1+maxn-minn)
    draw.circle(x[i], t[i], radius*circ)
  }
  x <- seq(0,1,by=.01)
  betabs <- as.vector(betabs)
  betaws <- as.vector(betaws)
  t <- matrix(ncol=length(x), nrow=length(betabs))
  for(i in 1:length(x)){
    t[,i] <- betabs*x[i] + betaws*(1-x[i])
  }
  et <- apply(t,2,mean)
  lines(x,et, col="yellow")
  lwr <- apply(t,2,function (x) quantile(x, probs=c(low)))
  upr <- apply(t,2,function (x) quantile(x, probs=c(up)))
  lines(x, lwr, col="red")
  lines(x, upr, col="red")
}
}

#xtfit(x,t,n,.04,"",ei.1$betabs, ei.1$betaws, .2,.8)


#XTfitg plot
.xtfitg <- function(ei.object){
  if(!("betabs"%in% names(ei.object))){
   message("Error: This plot function requires an ei.sim object.")
  }
  if("betabs"%in% names(ei.object)){
  ok <- !is.na(ei.object$betab)&!is.na(ei.object$betaw)
  x <- ei.object$x[ok]
  t <- ei.object$t[ok]
  n <- ei.object$n[ok]
  betabs <- ei.object$betabs[ok,]
  betaws <- ei.object$betaws[ok,]
  low <- .1
  up <- .9
  circ <- .04
  plot(x, t, xlim=c(0,1), ylim=c(0,1),xaxs="i",yaxs="i", main="X and T
Scatterplot with E(T|X), 80% CIs, and Goodman", ylab="T", xlab="X",
       pch=20)
  minn <- min(n)
  maxn <- max(n)
  for (i in 1:length(x)){
    radius = (n[i]-minn+1)/(1+maxn-minn)
    draw.circle(x[i], t[i], radius*circ)
  }
  x <- seq(0,1,by=.01)
  betabs <- as.vector(betabs)
  betaws <- as.vector(betaws)
  t <- matrix(ncol=length(x), nrow=length(betabs))
  for(i in 1:length(x)){
    t[,i] <- betabs*x[i] + betaws*(1-x[i])
  }
  et <- apply(t,2,mean)
  lines(x,et, col="yellow")
  lwr <- apply(t,2,function (x) quantile(x, probs=c(low)))
  upr <- apply(t,2,function (x) quantile(x, probs=c(up)))
  lines(x, lwr, col="red")
  lines(x, upr, col="red")
  t <- ei.object$t
  x <-ei.object$x
  lm.fit <- lm(t ~ x)
  abline(lm.fit, col="green")
}
}

#Goodman plot
.goodman <- function(ei.object){
  x <- ei.object$x
  t <- ei.object$t
  plot(x, t, xlim=c(0,1), ylim=c(0,1),xaxs="i",yaxs="i", main="X and T
Scatterplot with Goodman", ylab="T", xlab="X", pch=20)
  lm.fit <- lm(t ~ x)
  abline(lm.fit, col="red")
}


#Estsims plot

.estsims <- function(ei.object){
  if(!("betabs"%in% names(ei.object))){
   message("Error: This plot function requires an ei.sim object.")
  }
  if("betabs"%in% names(ei.object)){
  ok <- !is.na(ei.object$betab)&!is.na(ei.object$betaw)
  betabs <- ei.object$betabs[ok,]
  betaws <- ei.object$betaws[ok,]
  colors = runif(length(betabs),26,51)
  plot(betabs, betaws, xlim=c(0,1), ylim=c(0,1),xaxs="i",yaxs="i",
       main="Simulations of betaW and betaB", ylab="betaW
simulations", xlab="betaB simulations", pch=20, col=colors, lty=2,
       cex=.25)
}
}

#boundXB


.boundXb <- function(ei.object){
  x <- ei.object$x
  t <- ei.object$t
  n <- ei.object$n
  truebb <- ei.object$truth[,1]
  bounds <- bounds1(x, t, n)
  plot(x, truebb, xlim=c(0,1), ylim=c(0,1),xaxs="i",yaxs="i",
       main="Aggregation Bias for betaB", ylab="True betab", xlab="X",
       pch=20)
  for (i in 1:length(x)){
    lines(c(x[i], x[i]), c(bounds[,1][i], bounds[,2][i]))
  }
  lm.xb <- lm(truebb ~ x)
  abline(lm.xb, lty=2)
}

.boundXw <- function(ei.object){
  x <- ei.object$x
  t <- ei.object$t
  n <- ei.object$n
  truebw <- ei.object$truth[,2]
  bounds <- bounds1(x, t, n)
  plot(x, truebw, xlim=c(0,1), ylim=c(0,1),xaxs="i",yaxs="i",
       main="Aggregation Bias for betaW", ylab="True betaw", xlab="X",
       pch=20)
  for (i in 1:length(x)){
    lines(c(x[i], x[i]), c(bounds[,3][i], bounds[,4][i]))
  }
  lm.xw <- lm(truebw ~ x)
  abline(lm.xw, lty=2)
}


#truth
.truthfn <- function(ei.object){
  n <- ei.object$n
  x <- ei.object$x
  omx <- 1-x
  truebb <- ei.object$truth[,1]
  truebw <- ei.object$truth[,2]
  betabs <- ei.object$betabs
  betaws <- ei.object$betaws
  betab <- ei.object$betab
  betaw <- ei.object$betaw
  truthbb <- sum(truebb*n)/sum(n)
  truthbw <- sum(truebw*n)/sum(n)
  circ=.04
  par(mfrow=c(2,2))
  ag <- .aggs(ei.object)
  plot(density(ag[,1]),
       xlim=c(0,1),ylim=c(0,max(density(ag[,1])$y)+1),
       yaxs="i",xaxs="i", main="Density of Bb Posterior & Truth",
       xlab="Bb",ylab="Density")
  lines(c(truthbb, truthbb), c(0,.25*(max(density(ag[,1])$y)+1)),
        lwd=3)
  plot(density(ag[,2]),
       xlim=c(0,1),ylim=c(0,max(density(ag[,2])$y)+1), yaxs="i",
       xaxs="i", main="Density of Bw Posterior & Truth",
       xlab="Bw",ylab="Density")
  lines(c(truthbw, truthbw), c(0,.25*(max(density(ag[,2])$y)+1)),
        lwd=3)
  plot(betab, truebb, xlim=c(0,1),ylim=c(0,1),xaxs="i",yaxs="i",
       xlab="Estimated betab",cex=.1,ylab="True betab")
  minn <- min(x*n)
  maxn <- max(x*n)
  for (i in 1:length(betab)){
    radius = (n[i]*x[i]-minn+1)/(1+maxn-minn)
    draw.circle(betab[i], truebb[i], radius*circ)
  }
  ci80b = .CI80b(ei.object)
  low = mean(abs(ci80b[,1]-betab))
  high = mean(abs(ci80b[,2]-betab))
  abline(0,1)
  lines(c(0,1),c(-low,1-low),lty=2)
  lines(c(0,1),c(high,1+high),lty=2)
  plot(betaw, truebw,
       xlim=c(0,1),ylim=c(0,1),xaxs="i",yaxs="i",xlab="Estimated
betaw", ylab="True betaw",cex=.1)
  minn <- min(omx*n)
  maxn <- max(omx*n)
  for (i in 1:length(betaw)){
    radius = (omx[i]*n[i]-minn+1)/(1+maxn-minn)
    draw.circle(betaw[i], truebw[i], radius*circ)
  }
  ci80w = .CI80w(ei.object)
  low = mean(abs(ci80w[,1]-betaw))
  high = mean(abs(ci80w[,2]-betaw))
  abline(0,1)
  lines(c(0,1),c(-low,1-low),lty=2)
  lines(c(0,1),c(high,1+high),lty=2)
}


#Functions for Quantities of Interest

.betaB <- function(ei.object){
    if(!("betabs"%in% names(ei.object))){
   message("Error: This eiread function requires an ei.sim object.")
  }
  if("betabs"%in% names(ei.object)){
  ei.object$betab
}
}

.betaW <- function(ei.object){
   if(!("betabs"%in% names(ei.object))){
   message("Error: This eiread function requires an ei.sim object.")
  }
  if("betabs"%in% names(ei.object)){
  ei.object$betaw
}
}

.sbetab <- function(ei.object){
   if(!("betabs"%in% names(ei.object))){
   message("Error: This eiread function requires an ei.sim object.")
  }
  if("betabs"%in% names(ei.object)){
  ei.object$sbetab
}
 }

.sbetaw <- function(ei.object){
   if(!("betabs"%in% names(ei.object))){
   message("Error: This eiread function requires an ei.sim object.")
  }
  if("betabs"%in% names(ei.object)){
  ei.object$sbetaw
}
}
   

.phi <- function(ei.object){
  ei.object$phi
}

.psisims <- function(ei.object){
   if(!("betabs"%in% names(ei.object))){
   message("Error: This eiread function requires an ei.sim object.")
  }
  if("betabs"%in% names(ei.object)){
  ei.object$psi
}
}

.bounds <- function(ei.object){
  bounds1(ei.object$x, ei.object$t, ei.object$n)
}

.abounds <- function(ei.object){
  x <- ei.object$x
  t <- ei.object$t
  n <- ei.object$n
  bounds <- bounds1(x,t,n)
  omx <- 1-x
  Nb <- n*x
  Nw <- n*omx
  LAbetaB <- weighted.mean(bounds[,1],Nb)
  UAbetaB <- weighted.mean(bounds[,2], Nb)
  LAbetaW <- weighted.mean(bounds[,3], Nw)
  UAbetaW <- weighted.mean(bounds[,4], Nw)
  return(matrix(c(LAbetaB, UAbetaB, LAbetaW,UAbetaW), nrow=2))
}
	
.aggs <- function(ei.object){
   if(!("betabs"%in% names(ei.object))){
   message("Error: This eiread function requires an ei.sim object.")
  }
  if("betabs"%in% names(ei.object)){
  ok <- !is.na(ei.object$betab)&!is.na(ei.object$betaw)
  x <- ei.object$x[ok]
  t <- ei.object$t[ok]
  n <- ei.object$n[ok]
  betab <- ei.object$betabs[ok,]
  betaw <- ei.object$betaws[ok,]
  omx <- 1-x
  Nb <- n*x
  Nw <- n*omx
  Bbgg <- vector(mode="numeric", length=dim(betab)[2])
  for (i in 1:dim(betab)[2]){
    Bbgg[i] <- weighted.mean(betab[,i], Nb)
  }
  Bwgg <- vector(mode="numeric", length=dim(betaw)[2])
  for (i in 1:dim(betaw)[2]){
    Bwgg[i] <- weighted.mean(betaw[,i], Nw)
  }
  return(cbind(Bbgg, Bwgg))
}
}
	
.maggs <- function(ei.object){
   if(!("betabs"%in% names(ei.object))){
   message("Error: This eiread function requires an ei.sim object.")
  }
  if("betabs"%in% names(ei.object)){
  ok <- !is.na(ei.object$betab)&!is.na(ei.object$betaw)
  x <- ei.object$x[ok]
  t <- ei.object$t[ok]
  n <- ei.object$n[ok]
  betab <- ei.object$betabs[ok,]
  betaw <- ei.object$betaws[ok,]
  omx <- 1-x
  Nb <- n*x
  Nw <- n*omx
  Bbgg <- vector(mode="numeric", length=dim(betab)[2])
  for (i in 1:dim(betab)[2]){
    Bbgg[i] <- weighted.mean(betab[,i], Nb)
  }
  Bwgg <- vector(mode="numeric", length=dim(betaw)[2])
  for (i in 1:dim(betaw)[2]){
    Bwgg[i] <- weighted.mean(betaw[,i], Nw)
  }
  return(c(mean(Bbgg), mean(Bwgg), sd(Bbgg), sd(Bwgg)))
}
}
	
.VCaggs <- function(ei.object){
   if(!("betabs"%in% names(ei.object))){
   message("Error: This eiread function requires an ei.sim object.")
  }
  if("betabs"%in% names(ei.object)){
  ok <- !is.na(ei.object$betab)&!is.na(ei.object$betaw)
  x <- ei.object$x[ok]
  t <- ei.object$t[ok]
  n <- ei.object$n[ok]
  betab <- ei.object$betabs[ok,]
  betaw <- ei.object$betaws[ok,]
  omx <- 1-x
  Nb <- n*x
  Nw <- n*omx
  Bbgg <- vector(mode="numeric", length=dim(betab)[2])
  for (i in 1:dim(betab)[2]){
    Bbgg[i] <- weighted.mean(betab[,i], Nb)
  }
  Bwgg <- vector(mode="numeric", length=dim(betaw)[2])
  for (i in 1:dim(betaw)[2]){
    Bwgg[i] <- weighted.mean(betaw[,i], Nw)
  }
  vc <- matrix(c(var(Bbgg), cov(Bbgg, Bwgg), cov(Bbgg, Bwgg),
                 var(Bwgg, Bwgg)), nrow=2)
  return(vc)
}
}
	
.CI80b <- function(ei.object){
   if(!("betabs"%in% names(ei.object))){
   message("Error: This eiread function requires an ei.sim object.")
  }
  if("betabs"%in% names(ei.object)){
  ok <- !is.na(ei.object$betab)&!is.na(ei.object$betaw)
  betab <- ei.object$betabs[ok,]
  lwr <- vector(mode="numeric",length=length(ei.object$x))
  upr <- vector(mode="numeric",length=length(ei.object$x))
  lwr[ok] <- apply(betab, 1, function(x) quantile(x, probs=c(.1)))
  lwr[!ok] <- NA
  upr[ok] <-apply(betab, 1, function(x) quantile(x, probs=c(.9)))
  upr[!ok] <- NA
  return(cbind(lwr,upr))
}
}
	
.CI80w <- function(ei.object){
   if(!("betabs"%in% names(ei.object))){
   message("Error: This eiread function requires an ei.sim object.")
  }
  if("betabs"%in% names(ei.object)){
  ok <- !is.na(ei.object$betab)&!is.na(ei.object$betaw)
  betaw <- ei.object$betaws[ok,]
  lwr <- vector(mode="numeric",length=length(ei.object$x))
  upr <- vector(mode="numeric",length=length(ei.object$x))
  lwr[ok] <- apply(betaw, 1, function(x) quantile(x, probs=c(.1)))
  lwr[!ok] <- NA
  upr[ok] <-apply(betaw, 1, function(x) quantile(x, probs=c(.9)))
  upr[!ok] <- NA
  return(cbind(lwr,upr))
}
}
	
.eaggbias <- function(ei.object){
   if(!("betabs"%in% names(ei.object))){
   message("Error: This eiread function requires an ei.sim object.")
  }
  if("betabs"%in% names(ei.object)){
  x <- ei.object$x
  mbetab <- ei.object$betab
  mbetaw <- ei.object$betaw
  lm.b <- lm(mbetab ~ x)
  lm.w <- lm(mbetaw ~ x)
  output <-
    list(summary(lm.b)$coefficients,summary(lm.w)$coefficients)
  names(output) <- c("betaB", "betaW")
  return(output)
}
}
	
.goodman <- function(ei.object){
  x <- ei.object$x
  t <- ei.object$t
  lm.g <- lm(t ~ x)
  w <- 1-x
  lm.w <- lm(t ~ w)
  BetaW <- summary(lm.g)$coefficients[1,]
  BetaB <- summary(lm.w)$coefficients[1,]
  coefs <- rbind(BetaB, BetaW)
  return(coefs)
}

.movieD <- function(ei.object){
  x <- ei.object$x
  t <- ei.object$t
  n <- ei.object$n
  bounds <- bounds1(x,t,n)
  bbounds <- cbind(bounds[,1], bounds[,2])
  wbounds <- cbind(bounds[,4], bounds[,3])
  n <- dim(bounds)[1]
  plot(c(100,200), xlim=c(0,1), ylim=c(0,1), col="white",
       ylab="betaW", xlab="betaB", xaxs="i",yaxs="i",
       main="Tomography Plot")
  input<-0
  while(input!="s"){
    input<-tomogonce(input,last.input)
    last.input<-input
    input<-getinput()
  }
}
getinput <- function(){
  readline("Hit <enter> for next observation, enter observation number, or hit <s> to stop: ")
}

.tomogonce <- function(input,last.input){
  par(mfrow=c(1,1))
  plot(c(100,200), xlim=c(0,1), ylim=c(0,1), col="white",
       ylab="betaW", xlab="betaB", xaxs="i",yaxs="i",
       main="Tomography Plot")
  if(input==""){ #input is <enter>, plot next observation
    last.input<-as.integer(last.input)
    input<-last.input+1
    for(i in 1:n){
      lines(bbounds[i,], wbounds[i, ], col="yellow")
    }  
    lines(bbounds[input,], wbounds[input,], col="black")
  }
  else{ #input is observation number
    input<-as.integer(input)
    for(i in 1:n){
      lines(bbounds[i,], wbounds[i,], col="yellow")
    }
    lines(bbounds[input,], wbounds[input,], col="black")
  }
  return(input)
}





#Movie plots

.movieD <- function(ei.object){
  x <- ei.object$x
  t <- ei.object$t
  n <- ei.object$n
  bounds <- bounds1(x,t,n)
  bbounds <- cbind(bounds[,1], bounds[,2])
  wbounds <- cbind(bounds[,4], bounds[,3])
  n <- dim(bounds)[1]
  plot(c(100,200), xlim=c(0,1), ylim=c(0,1), col="white",
       ylab="betaW", xlab="betaB", xaxs="i",yaxs="i",
       main="Tomography Plot")
  input<-0
  while(input!="s"){
    input<-.tomogonce(input,last.input, ei.object)
    last.input<-input
    input<-.getinput()
  }
}
.getinput <- function(){
  readline("Hit <enter> for next observation, enter observation number, or hit <s> to stop: ")
}

.tomogonce <- function(input,last.input, ei.object){
    x <- ei.object$x
  t <- ei.object$t
  n <- ei.object$n
  bounds <- bounds1(x,t,n)
  bbounds <- cbind(bounds[,1], bounds[,2])
  wbounds <- cbind(bounds[,4], bounds[,3])
  n <- dim(bounds)[1]
  par(mfrow=c(1,1))
  plot(c(100,200), xlim=c(0,1), ylim=c(0,1), col="white",
       ylab="betaW", xlab="betaB", xaxs="i",yaxs="i",
       main="Tomography Plot")
  if(input==""){ #input is <enter>, plot next observation
    last.input<-as.integer(last.input)
    input<-last.input+1
    for(i in 1:n){
      lines(bbounds[i,], wbounds[i, ], col="yellow")
    }  
    lines(bbounds[input,], wbounds[input,], col="black")
  }
  else{ #input is observation number
    input<-as.integer(input)
    for(i in 1:n){
      lines(bbounds[i,], wbounds[i,], col="yellow")
    }
    lines(bbounds[input,], wbounds[input,], col="black")
  }
  return(input)
}


.getinput <- function(){
  readline("Hit <enter> for next observation, enter observation number, or hit <s> to stop: ")
}

.postonce<-function(input,last.input,betab,betaw,betabs,betaws){

  if(input==""){ #input is <enter>, will plot next observation
    last.input<-as.integer(last.input)
    input<-last.input+1
  }
  else{ #input is observation number
    input<-as.integer(input) 
  }

  par(mfrow=c(2,2), oma=c(0,0,2,0))

  # plot posterior distribution of precinct parameters
  plot(density(betab[input,]),
    xlim=c(0,1),ylim=c(0,max(density(betab[input,])$y)+1),
    yaxs="i",xaxs="i", main="Posterior Distribution of betaB",
    xlab="Bb",ylab="Density")
  lines(c(0,.25*(max(density(betab[input,])$y)+1)),lwd=3)
  plot(density(betaw[input,]),
    xlim=c(0,1),ylim=c(0,max(density(betaw[input,])$y)+1),
    yaxs="i",xaxs="i", main="Posterior Distribution of betaW",
    xlab="Bw",ylab="Density")
  lines(c(0,.25*(max(density(betaw[input,])$y)+1)),lwd=3)
  
# plot simulated values of betaB and betaW of precinct
  colors = runif(length(betabs),26,51)
  plot(betabs[input,], betaws[input,], xlim=c(0,1), ylim=c(0,1),xaxs="i",
    yaxs="i",main="Simulations of betaW and betaB",
    ylab="betaW simulations", xlab="betaB simulations",
    pch=20,col=colors,lty=2,cex=.25)

  mtext(sprintf("Plots for Observation %d", input),line=0.5,outer=TRUE)

return(input)

}

.movie <- function(ei.object){
  ok <- !is.na(ei.object$betab)
  betabs <- ei.object$betabs[ok,]
  ok <- !is.na(ei.object$betaw)
  betaws <- ei.object$betaws[ok,]

  betab <- ei.object$betabs
  betaw <- ei.object$betaws
  
 input<-1 #initialize at precinct 1
 last.input<-0
 while(input!="s"){
    input<-.postonce(input,last.input,betab,betaw,betabs,betaws)
    last.input<-input
    input<-.getinput()
 }
   
}



#EI RxC Plots

#########
##Heat plot for the eiRxC case
#########

#form = formula
#total = totals for each precinct
#data = data
#names = Names of groups in order: X-axis, Y-axis, Other
#covariate = extra group or covariate to create colors in the plot for
tomogRxC <- function(formula, data, total=NULL, refine=100){
	#require(sp)  Now specified as Import
    noinfocount = 0
	form <- formula
    ##total <- dbuf$total
    #data <- dbuf$data
    dvname <- terms.formula(formula)[[2]]
    covariate <- NA
	#Make the bounds
	rows <- c(all.names(form)[6:(length(all.names(form)))])
    names=rows
    cols <- c(all.names(form)[3])
#    if(sum(data[,rows][,1]<1.1)==length(data[,rows][,1])){
#       data <- round(data*data[,total])
#}
	#print(data[,cols])
    options(warn=-1)
	bnds <- bounds(form, data=data,rows=rows, column =cols,threshold=0)
    options(warn=0)
	#Totals
	dv <- data[, all.names(form)[3]]
	#Assign other category
	bndsoth <- bnds$bounds[[3]]
	oth <- data[,all.names(form)[length(all.names(form))]]

	#Assign x-axis category
	bndsx <- bnds$bounds[[1]]
	xcat <- data[,all.names(form)[6]]

	#Assign y-axis category
	bndsy <- bnds$bounds[[2]]
	ycat <- data[,all.names(form)[7]]

	#Minimums & Maximums
	minx <- bndsx[,1]
	miny <- bndsy[,1]
	minoth <- bndsoth[,1]
	maxx <- bndsx[,2]
	maxy <- bndsy[,2]
	maxoth <- bndsoth[,2]

	#####
	#Starting point when x is at minimum
	##
	#Holding x at its minimum, what are the bounds on y?
	
	#When x is at its minimum, the new dv and total are:
	newdv <- dv - (minx*xcat)
	newtot <- oth + ycat 
	t <- newdv/newtot
	y <- ycat/newtot
	
	#The new bounds on the y category are:

	lby <- cbind(miny, (t - maxoth*oth/newtot)/(y))
	lby[,2] <- ifelse(y==0, 0, lby[,2])
	lowy <- apply(lby,1,max)
	hby <- cbind((t-minoth*oth/newtot)/y,maxy)
	highy <- apply(hby,1,min)

	#####
	#Starting point when x is at maximum
	##
	#Holding x at its maximum, what are the bounds on y?

	#The new bounds on x are:
	newtot <- oth + xcat
	newdv <- dv - (miny*ycat)
	x <- xcat/newtot
	t <- newdv/newtot
	lbx <- cbind(minx, (t-maxoth*oth/newtot)/x)
	lbx[,2] <- ifelse(x==0, 0, lbx[,2])
	lowx <- apply(lbx,1,max)
	hbx <- cbind((t-minoth*oth/newtot)/x,maxx)
	highx <- apply(hbx,1,min)

	#Graph starting points
	#High starting points
	hstr <- cbind(minx, highy)
	#High ending points
	hend <- cbind(highx, miny)

	#Low starting points
	lstr <- cbind(minx, lowy)
	lend <- cbind(lowx, miny)

	xl <- paste("Percent", names[1], dvname[2])
	yl <- paste("Percent", names[2], dvname[2])
	mn <- paste("Tomography Plot in a 2x3 Table (", names[3], " Other Category)*", sep="")

plot(c(0,0), xlim=c(0,1), ylim=c(0,1), xaxs="i", yaxs="i",xlab=xl, ylab=yl, col="white", main=mn)


ok <- !is.na(hstr[,2]) & !is.na(hend[,1])
exp1 <- hstr[ok,2]>=maxy[ok]
exp2 <- hend[ok,1]>=maxx[ok]
exp3 <- lstr[ok,2]<=miny[ok]
exp4 <- lend[ok,1]<=minx[ok]
hstr <- hstr[ok,]
hend <- hend[ok,]
lstr <- lstr[ok,]
lend <- lend[ok,]
dv <- dv[ok]
ycat <- ycat[ok]
oth <- oth[ok]
minoth <- minoth[ok]
xcat <- xcat[ok]
maxy <- maxy[ok]
maxx <- maxx[ok]
contourx <- seq(0,1,by=1/refine)
contoury <- seq(0,1,by=1/refine)
contourz <- matrix(0,nrow=length(contourx), ncol=length(contoury))
for(i in 1:dim(hstr)[1]){
	if((exp1[i] + exp2[i] + exp3[i] + exp4[i])==0){
		xaxs <- c(hstr[i,1],  lstr[i,1],lend[i,1],hend[i,1])
		yaxs <- c(hstr[i,2], lstr[i,2],lend[i,2], hend[i,2])
		side1 <- c(xaxs,xaxs[1])
		side2 <- c(yaxs,yaxs[1])
		c1 <- sum(side1[1:(length(side1)-1)]*side2[2:length(side2)])
		c2 <- sum(side1[2:(length(side1))]*side2[1:(length(side2)-1)])
		area <- abs(c1-c2)/2
        if(area==1){
            noinfocount <- noinfocount+1
        }
		for(j in 1:length(as.vector(contourx))){
			contourz[j,] <- contourz[j,] + ifelse(point.in.polygon(rep(contourx[j], length(contourx)), contoury, xaxs, yaxs)==1,1+1/(.1+area),0)
        
	}
	}
	if((exp1[i]==1) & (exp2[i])==0){
		cut <- (dv[i]-(oth[i])*minoth[i])/xcat[i] - maxy[i]*ycat[i]/xcat[i]
		kink1x <- c(cut)
		kink1y <- c(maxy[i])
		}
	if((exp2[i]==1) & (exp1[i])==0){
		cut <- (dv[i]-(oth[i])*minoth[i])/ycat[i] - maxx[i]*xcat[i]/ycat[i]
		kink1x <- c(maxx[i])
		kink1y <- c(cut)
	}
	if((exp2[i]==1 & exp1[i]==1)){
		cut <- (dv[i]-(oth[i])*minoth[i])/ycat[i] - maxx[i]*xcat[i]/ycat[i]
		cut2 <- (dv[i]-(oth[i])*minoth[i])/xcat[i] - maxy[i]*ycat[i]/xcat[i]
		kink1x <- c(maxx[i], cut2)
		kink1y <- c(cut, maxy[i])
	}
	if((exp3[i]==1) & (exp4[i])==0){
		cut <- (dv[i]-(oth[i])*maxoth[i])/xcat[i] - miny[i]*ycat[i]/xcat[i]
		kink2x <- c(cut)
		kink2y <- c(miny[i])
		}
	if((exp4[i]==1) & (exp3[i])==0){
		cut <- (dv[i]-(oth[i])*maxoth[i])/ycat[i] - minx[i]*xcat[i]/ycat[i]
		kink2x <- c(minx[i])
		kink2y <- c(cut)
	}
	if((exp3[i]==1 & exp4[i]==1)){
		cut <- (dv[i]-(oth[i])*maxoth[i])/ycat[i] - minx[i]*xcat[i]/ycat[i]
		cut2 <- (dv[i]-(oth[i])*maxoth[i])/xcat[i] - miny[i]*ycat[i]/xcat[i]
		kink2x <- c(minx[i], cut2)
		kink2y <- c(cut, miny[i])
	}
	if((exp3[i] + exp4[i])==0 & (exp1[i] + exp2[i] + exp3[i] + exp4[i])!=0){
		xaxs <- c(hstr[i,1],  lstr[i,1],lend[i,1],hend[i,1], kink1x)
		xaxs <- ifelse(is.nan(xaxs), 0, xaxs)
		yaxs <- c(hstr[i,2], lstr[i,2],lend[i,2], hend[i,2], kink1y)
		yaxs <- ifelse(is.nan(yaxs), 0, yaxs)
		side1 <- c(xaxs,xaxs[1])
		side2 <- c(yaxs,yaxs[1])
	    c1 <- sum(side1[1:(length(side1)-1)]*side2[2:length(side2)])
		c2 <- sum(side1[2:(length(side1))]*side2[1:(length(side2)-1)])
		area <- abs(c1-c2)/2
        if(area==1){
            noinfocount <- noinfocount+1
        }
		for(j in 1:length(as.vector(contourx))){
			contourz[j,] <- contourz[j,] + ifelse(point.in.polygon(rep(contourx[j], length(contourx)), contoury, xaxs, yaxs)==1,1+1/(.1+area),0)
         
	}
		
	}
	if((exp1[i] + exp2[i])==0 & (exp1[i] + exp2[i] + exp3[i] + exp4[i])!=0){
		xaxs <- c(hstr[i,1],  lstr[i,1],kink2x,lend[i,1],hend[i,1])
		xaxs <- ifelse(is.nan(xaxs), 0, xaxs)
		yaxs <- c(hstr[i,2], lstr[i,2],kink2y,lend[i,2], hend[i,2])
		yaxs <- ifelse(is.nan(yaxs), 0, yaxs)
		side1 <- c(xaxs,xaxs[1])
		side2 <- c(yaxs,yaxs[1])
		c1 <- sum(side1[1:(length(side1)-1)]*side2[2:length(side2)])
		c2 <- sum(side1[2:(length(side1))]*side2[1:(length(side2)-1)])
		area <- abs(c1-c2)/2
        if(area==1){
            noinfocount <- noinfocount+1
        }
		for(j in 1:length(as.vector(contourx))){
			contourz[j,] <- contourz[j,] + ifelse(point.in.polygon(rep(contourx[j], length(contourx)), contoury, xaxs, yaxs)==1,1+1/(.1+area),0)
	}	}
	if((exp1[i] + exp2[i])!=0 & (exp3[i] + exp4[i])!=0){
		xaxs <- c(hstr[i,1],  lstr[i,1],kink2x,lend[i,1],hend[i,1], kink1x)
		xaxs <- ifelse(is.nan(xaxs), 0, xaxs)
		yaxs <- c(hstr[i,2], lstr[i,2],kink2y,lend[i,2], hend[i,2], kink1y)
		yaxs <- ifelse(is.nan(yaxs), 0, yaxs)
		side1 <- c(xaxs,xaxs[1])
		side2 <- c(yaxs,yaxs[1])
		c1 <- sum(side1[1:(length(side1)-1)]*side2[2:length(side2)])
		c2 <- sum(side1[2:(length(side1))]*side2[1:(length(side2)-1)])
		area <- abs(c1-c2)/2
        if(area==1){
            noinfocount <- noinfocount+1
        }
		for(j in 1:length(as.vector(contourx))){
			contourz[j,] <- contourz[j,] + ifelse(point.in.polygon(rep(contourx[j], length(contourx)), contoury, xaxs, yaxs)==1,1+1/(.1+area),0)
	}
	}
	}
image(contourx[2:refine], contoury[2:refine], contourz[2:refine,2:refine], , col=sort(heat.colors(refine), decreasing=T), xlab=xl, ylab=yl, main=mn, xlim=c(0,1), add=T)
for(i in 1:dim(hstr)[1]){
	if((exp1[i] + exp2[i] + exp3[i] + exp4[i])==0){
		xaxs <- c(hstr[i,1],  lstr[i,1],lend[i,1],hend[i,1])
		yaxs <- c(hstr[i,2], lstr[i,2],lend[i,2], hend[i,2])
		side1 <- c(xaxs,xaxs[1])
		side2 <- c(yaxs,yaxs[1])
		c1 <- sum(side1[1:(length(side1)-1)]*side2[2:length(side2)])
		c2 <- sum(side1[2:(length(side1))]*side2[1:(length(side2)-1)])
		area <- abs(c1-c2)/2
        polygon(xaxs,yaxs, col=rgb(0,0,0,alpha=0), border="black", lty=1, lwd=.25)
	}
	if((exp1[i]==1) & (exp2[i])==0){
		cut <- (dv[i]-(oth[i])*minoth[i])/xcat[i] - maxy[i]*ycat[i]/xcat[i]
		kink1x <- c(cut)
		kink1y <- c(maxy[i])
		}
	if((exp2[i]==1) & (exp1[i])==0){
		cut <- (dv[i]-(oth[i])*minoth[i])/ycat[i] - maxx[i]*xcat[i]/ycat[i]
		kink1x <- c(maxx[i])
		kink1y <- c(cut)
	}
	if((exp2[i]==1 & exp1[i]==1)){
		cut <- (dv[i]-(oth[i])*minoth[i])/ycat[i] - maxx[i]*xcat[i]/ycat[i]
		cut2 <- (dv[i]-(oth[i])*minoth[i])/xcat[i] - maxy[i]*ycat[i]/xcat[i]
		kink1x <- c(maxx[i], cut2)
		kink1y <- c(cut, maxy[i])
	}
	if((exp3[i]==1) & (exp4[i])==0){
		cut <- (dv[i]-(oth[i])*maxoth[i])/xcat[i] - miny[i]*ycat[i]/xcat[i]
		kink2x <- c(cut)
		kink2y <- c(miny[i])
		}
	if((exp4[i]==1) & (exp3[i])==0){
		cut <- (dv[i]-(oth[i])*maxoth[i])/ycat[i] - minx[i]*xcat[i]/ycat[i]
		kink2x <- c(minx[i])
		kink2y <- c(cut)
	}
	if((exp3[i]==1 & exp4[i]==1)){
		cut <- (dv[i]-(oth[i])*maxoth[i])/ycat[i] - minx[i]*xcat[i]/ycat[i]
		cut2 <- (dv[i]-(oth[i])*maxoth[i])/xcat[i] - miny[i]*ycat[i]/xcat[i]
		kink2x <- c(minx[i], cut2)
		kink2y <- c(cut, miny[i])
	}
	if((exp3[i] + exp4[i])==0 & (exp1[i] + exp2[i] + exp3[i] + exp4[i])!=0){
		xaxs <- c(hstr[i,1],  lstr[i,1],lend[i,1],hend[i,1], kink1x)
		xaxs <- ifelse(is.nan(xaxs), 0, xaxs)
		yaxs <- c(hstr[i,2], lstr[i,2],lend[i,2], hend[i,2], kink1y)
		yaxs <- ifelse(is.nan(yaxs), 0, yaxs)
		side1 <- c(xaxs,xaxs[1])
		side2 <- c(yaxs,yaxs[1])
	    c1 <- sum(side1[1:(length(side1)-1)]*side2[2:length(side2)])
		c2 <- sum(side1[2:(length(side1))]*side2[1:(length(side2)-1)])
		area <- abs(c1-c2)/2
        polygon(xaxs,yaxs, col=rgb(0,0,0,alpha=0), border="black", lty=1, lwd=.25)
	}
	if((exp1[i] + exp2[i])==0 & (exp1[i] + exp2[i] + exp3[i] + exp4[i])!=0){
		xaxs <- c(hstr[i,1],  lstr[i,1],kink2x,lend[i,1],hend[i,1])
		xaxs <- ifelse(is.nan(xaxs), 0, xaxs)
		yaxs <- c(hstr[i,2], lstr[i,2],kink2y,lend[i,2], hend[i,2])
		yaxs <- ifelse(is.nan(yaxs), 0, yaxs)
		side1 <- c(xaxs,xaxs[1])
		side2 <- c(yaxs,yaxs[1])
		c1 <- sum(side1[1:(length(side1)-1)]*side2[2:length(side2)])
		c2 <- sum(side1[2:(length(side1))]*side2[1:(length(side2)-1)])
		area <- abs(c1-c2)/2
        polygon(xaxs,yaxs, col=rgb(0,0,0,alpha=0), border="black", lty=1, lwd=.25)
	}
	if((exp1[i] + exp2[i])!=0 & (exp3[i] + exp4[i])!=0){
		xaxs <- c(hstr[i,1],  lstr[i,1],kink2x,lend[i,1],hend[i,1], kink1x)
		xaxs <- ifelse(is.nan(xaxs), 0, xaxs)
		yaxs <- c(hstr[i,2], lstr[i,2],kink2y,lend[i,2], hend[i,2], kink1y)
		yaxs <- ifelse(is.nan(yaxs), 0, yaxs)
		side1 <- c(xaxs,xaxs[1])
		side2 <- c(yaxs,yaxs[1])
		c1 <- sum(side1[1:(length(side1)-1)]*side2[2:length(side2)])
		c2 <- sum(side1[2:(length(side1))]*side2[1:(length(side2)-1)])
		area <- abs(c1-c2)/2
        polygon(xaxs,yaxs, col=rgb(0,0,0,alpha=0), border="black", lty=1, lwd=.225)
	}
	}
message(paste("There are", noinfocount, "tomography polygons with no information"))
#contour(contourx[2:refine], contoury[2:refine], contourz[2:refine,2:refine], nlevels=nrow(data), method="simple", col="black", add=TRUE, vfont = c("sans serif", "plain"), drawlabels=F)
par(xpd=NA)
text(.78,-.18, paste("*There are", noinfocount, "tomography polygons with no information"), cex=.75)
par(xpd=FALSE)
	}


############
##Bounds plot for eiRxC case
############

#form = formula
#total = totals for each precinct
#data = data
#names = Names of groups in order: X-axis, Y-axis, Other
#covariate = extra group or covariate to create colors in the plot for
.bndplot <- function(dbuf){
    #require(sp)  Now specified as Import
	form <- dbuf$formula
    total <- dbuf$total
    data <- dbuf$data
    n <- nrow(data)
    print(n)
    covariate <- NA
	#Make the bounds
	rows <- c(all.names(form)[6:(length(all.names(form)))])
    names=rows
    cols <- c(all.names(form)[3])
    if(sum(data[,rows][,1]<1.1)==length(data[,rows][,1])){
        data <- round(data*data[,total])
}
	bnds <- bounds(form, data=data, rows=rows, column =cols,threshold=0)
    
	#Totals
	dv <- data[, all.names(form)[3]]

	#Assign other category
	bndsoth <- bnds$bounds[[3]]
	oth <- data[,all.names(form)[length(all.names(form))]]

	#Assign x-axis category
	bndsx <- bnds$bounds[[1]]
	xcat <- data[,all.names(form)[6]]

	#Assign y-axis category
	bndsy <- bnds$bounds[[2]]
	ycat <- data[,all.names(form)[7]]

	#Minimums & Maximums
	minx <- bndsx[,1]
	miny <- bndsy[,1]
	minoth <- bndsoth[,1]
	maxx <- bndsx[,2]
	maxy <- bndsy[,2]
	maxoth <- bndsoth[,2]

	#####
	#Starting point when x is at minimum
	##
	#Holding x at its minimum, what are the bounds on y?
	
	#When x is at its minimum, the new dv and total are:
	newdv <- dv - (minx*xcat)
	newtot <- oth + ycat 
	t <- newdv/newtot
	y <- ycat/newtot
	
	#The new bounds on the y category are:
	lby <- cbind(miny, (t - maxoth*oth/newtot)/(y))
	lby[,2] <- ifelse(y==0, 0, lby[,2])
	lowy <- apply(lby,1,max)
	hby <- cbind((t-minoth*oth/newtot)/y,maxy)
	highy <- apply(hby,1,min)

	####
	#Starting point when x is at maximum
	##
	#Holding x at its maximum, what are the bounds on y?
	#The new bounds on x are:
	newtot <- oth + xcat
	newdv <- dv - (miny*ycat)
	x <- xcat/newtot
	t <- newdv/newtot
	lbx <- cbind(minx, (t-maxoth*oth/newtot)/x)
	lbx[,2] <- ifelse(x==0, 0, lbx[,2])
	lowx <- apply(lbx,1,max)
	hbx <- cbind((t-minoth*oth/newtot)/x,maxx)
	highx <- apply(hbx,1,min)

	###
	#Graph starting points
	####

	#High starting points
	hstr <- cbind(minx, highy)
	#High ending points
	hend <- cbind(highx, miny)

	#Low starting points
	lstr <- cbind(minx, lowy)
	lend <- cbind(lowx, miny)
	
	#Colors for covariates
	if(!is.na(covariate)){
		redg <- data[,covariate]/(oth-data[,covariate])/max(data[,covariate]/(oth-data[,covariate]))
		blug <- 1-redg
	}
	if(is.na(covariate)){
		redg <- rep(.5, length(minx))
		blug <- rep(.5, length(minx))	
	}
	
	#Graph labels
	xl <- paste("Percent", names[1], "Vote Democrat")
	yl <- paste("Percent", names[2], "Vote Democrat")
	mn <- paste("Tomography Plot in a 2x3 Table (", names[3], " Other Category)", sep="")
	
	#Initial plot
	plot(c(0,0), xlim=c(0,1), ylim=c(0,1), xaxs="i", yaxs="i",xlab=xl, ylab=yl, col="white", main=mn)

	#Only non-NA starting pts are OK
	ok <- !is.na(hstr[,2]) & !is.na(hend[,1])
	
	#All different types of polygons
	exp1 <- hstr[ok,2]>=maxy[ok]
	exp2 <- hend[ok,1]>=maxx[ok]
	exp3 <- lstr[ok,2]<=miny[ok]
	exp4 <- lend[ok,1]<=minx[ok]
	
	#Subsets all the variables
	hstr <- hstr[ok,]
	hend <- hend[ok,]
	lstr <- lstr[ok,]
	lend <- lend[ok,]
	dv <- dv[ok]
	ycat <- ycat[ok]
	oth <- oth[ok]
	minoth <- minoth[ok]
	xcat <- xcat[ok]
	maxy <- maxy[ok]
	maxx <- maxx[ok]
	redg <- redg[ok]
	blug <- blug[ok]
	
for(i in 1:dim(hstr)[1]){
	#4 corner polygon
	if((exp1[i] + exp2[i] + exp3[i] + exp4[i])==0){
		xaxs <- c(hstr[i,1],  lstr[i,1],lend[i,1],hend[i,1])
		yaxs <- c(hstr[i,2], lstr[i,2],lend[i,2], hend[i,2])
		side1 <- c(xaxs,xaxs[1])
		side2 <- c(yaxs,yaxs[1])
		c1 <- sum(side1[1:(length(side1)-1)]*side2[2:length(side2)])
		c2 <- sum(side1[2:(length(side1))]*side2[1:(length(side2)-1)])
		area <- abs(c1-c2)/2
		#if(area>0 & !is.nan(area)){alpha = 1/(1+b*area)}
		#if(area>0 & !is.nan(area)){alpha = .5-.5*area}
		#if(area>0 & !is.nan(area)){alpha = ((1-area)^(3)+.2)*.83}
		if(area>0 & !is.nan(area)){alpha = min(.5/(area*(n)),1)}
		if(area==0 | is.nan(area)){alpha=.05}
		border = alpha 
		polygon(xaxs,yaxs, col=rgb(redg[i],0,blug[i],alpha=alpha), border=rgb(redg[i],0,blug[i],alpha=1), lty=2)
	}
	
	#Create more corners
	if((exp1[i]==1) & (exp2[i])==0){
		cut <- (dv[i]-(oth[i])*minoth[i])/xcat[i] - maxy[i]*ycat[i]/xcat[i]
		kink1x <- c(cut)
		kink1y <- c(maxy[i])
		}
	if((exp2[i]==1) & (exp1[i])==0){
		cut <- (dv[i]-(oth[i])*minoth[i])/ycat[i] - maxx[i]*xcat[i]/ycat[i]
		kink1x <- c(maxx[i])
		kink1y <- c(cut)
	}
	if((exp2[i]==1 & exp1[i]==1)){
		cut <- (dv[i]-(oth[i])*minoth[i])/ycat[i] - maxx[i]*xcat[i]/ycat[i]
		cut2 <- (dv[i]-(oth[i])*minoth[i])/xcat[i] - maxy[i]*ycat[i]/xcat[i]
		kink1x <- c(maxx[i], cut2)
		kink1y <- c(cut, maxy[i])
	}
	if((exp3[i]==1) & (exp4[i])==0){
		cut <- (dv[i]-(oth[i])*maxoth[i])/xcat[i] - miny[i]*ycat[i]/xcat[i]
		kink2x <- c(cut)
		kink2y <- c(miny[i])
		}
	if((exp4[i]==1) & (exp3[i])==0){
		cut <- (dv[i]-(oth[i])*maxoth[i])/ycat[i] - minx[i]*xcat[i]/ycat[i]
		kink2x <- c(minx[i])
		kink2y <- c(cut)
	}
	if((exp3[i]==1 & exp4[i]==1)){
		cut <- (dv[i]-(oth[i])*maxoth[i])/ycat[i] - minx[i]*xcat[i]/ycat[i]
		cut2 <- (dv[i]-(oth[i])*maxoth[i])/xcat[i] - miny[i]*ycat[i]/xcat[i]
		kink2x <- c(minx[i], cut2)
		kink2y <- c(cut, miny[i])
	}
	
	#Plot 5-sided polygon
	if((exp3[i] + exp4[i])==0 & (exp1[i] + exp2[i] + exp3[i] + exp4[i])!=0){
		xaxs <- c(hstr[i,1],  lstr[i,1],lend[i,1],hend[i,1], kink1x)
		xaxs <- ifelse(is.nan(xaxs), 0, xaxs)
		yaxs <- c(hstr[i,2], lstr[i,2],lend[i,2], hend[i,2], kink1y)
		yaxs <- ifelse(is.nan(yaxs), 0, yaxs)
		side1 <- c(xaxs,xaxs[1])
		side2 <- c(yaxs,yaxs[1])
	c1 <- sum(side1[1:(length(side1)-1)]*side2[2:length(side2)])
		c2 <- sum(side1[2:(length(side1))]*side2[1:(length(side2)-1)])
		area <- abs(c1-c2)/2
		#if(area>0 & !is.nan(area)){alpha = 1/(1+b*area)}
		if(area>0 & !is.nan(area)){alpha = min(.5/(area*(n)),1)}
		#if(area>0 & !is.nan(area)){alpha = ((1-area)^(3)+.2)*.53}
		if(area==0 | is.nan(area)){alpha=.05}
		border = alpha 
		polygon(xaxs,yaxs, col=rgb(redg[i],0,blug[i],alpha=alpha), border=rgb(redg[i],0,blug[i],alpha=1), lty=2)
		
	}
	
	#Another 5-sided polygon
	if((exp1[i] + exp2[i])==0 & (exp1[i] + exp2[i] + exp3[i] + exp4[i])!=0){
		xaxs <- c(hstr[i,1],  lstr[i,1],kink2x,lend[i,1],hend[i,1])
		xaxs <- ifelse(is.nan(xaxs), 0, xaxs)
		yaxs <- c(hstr[i,2], lstr[i,2],kink2y,lend[i,2], hend[i,2])
		yaxs <- ifelse(is.nan(yaxs), 0, yaxs)
		side1 <- c(xaxs,xaxs[1])
		side2 <- c(yaxs,yaxs[1])
		c1 <- sum(side1[1:(length(side1)-1)]*side2[2:length(side2)])
		c2 <- sum(side1[2:(length(side1))]*side2[1:(length(side2)-1)])
		area <- abs(c1-c2)/2
		if(area>0 & !is.nan(area)){alpha = min(.5/(area*(n)),1)}
		#if(area>0 & !is.nan(area)){alpha = ((1-area)^(3)+.2)*.53}
		if(area==0 | is.nan(area)){alpha=.05}
		border = alpha 
		polygon(xaxs,yaxs, col=rgb(redg[i],0,blug[i],alpha=alpha), border=rgb(redg[i],0,blug[i],alpha=1), lty=2)
	}
	
	#Plot 6-sided polygons
	if((exp1[i] + exp2[i])!=0 & (exp3[i] + exp4[i])!=0){
		xaxs <- c(hstr[i,1],  lstr[i,1],kink2x,lend[i,1],hend[i,1], kink1x)
		xaxs <- ifelse(is.nan(xaxs), 0, xaxs)
		yaxs <- c(hstr[i,2], lstr[i,2],kink2y,lend[i,2], hend[i,2], kink1y)
		yaxs <- ifelse(is.nan(yaxs), 0, yaxs)
		side1 <- c(xaxs,xaxs[1])
		side2 <- c(yaxs,yaxs[1])
		c1 <- sum(side1[1:(length(side1)-1)]*side2[2:length(side2)])
		c2 <- sum(side1[2:(length(side1))]*side2[1:(length(side2)-1)])
		area <- abs(c1-c2)/2
		#if(area>0 & !is.nan(area)){alpha = 1/(1+b*area)}
		if(area>0 & !is.nan(area)){alpha = min(.5/(area*(n)),1)}
		#if(area>0 & !is.nan(area)){alpha = ((1-area)^(3)+.2)*.53}	
		if(area==0 | is.nan(area)){alpha=.05}
		border = alpha 
		polygon(xaxs,yaxs, col=rgb(redg[i],0,blug[i],alpha=alpha), border=rgb(redg[i],0,blug[i],alpha=1), lty=2)
		}
	}
}


	

#Maximum likelihood estimation
#for
#t,x,n,Zb,Zw,data,erho,esigma,ebeta,ealphab,ealphaw,truth,precision
#see documentation

#@Rfun specifies function used to calculate R in the likelihood

  ei <- function(formula, total=NULL, Zb=1,Zw=1, id=NA, data=NA, erho=.5, esigma=.5,
               ebeta=.5, ealphab=NA, ealphaw=NA, truth=NA, simulate=TRUE,covariate=NULL, lambda1=4, lambda2=2, covariate.prior.list=NULL, tune.list=NULL, start.list=NULL, sample=1000, thin=1, burnin=1000, verbose=0, ret.beta="r", ret.mcmc=TRUE, usrfun=NULL){
    #Extract formula
    dv <- terms.formula(formula)[[2]]
    iv <- terms.formula(formula)[[3]]
    t <- as.character(dv)
    x <- as.character(iv)
    n <- as.character(total)
    id <- as.character(id)
    
    if(length(dv)==1){
    print("Running 2x2 ei")

    if(simulate==FALSE){
    dbuf <- ei.estimate(t,x, n,id=id,data=data, Zb=Zb, Zw=Zw, erho=erho, esigma=esigma, ebeta=ebeta, ealphab=ealphab, ealphaw=ealphaw, truth=truth)
    return(dbuf)
}
    if(simulate==TRUE){
    #If the table is two by two, use ei
    dbuf <- tryCatch(tryCatch(ei.estimate(t,x, n,id=id,
                        data=data, Zb=Zb, Zw=Zw, erho=erho,
                                          esigma=esigma, ebeta=ebeta,
                                          ealphab=ealphab,
                                          ealphaw=ealphaw,
                                          truth=truth),
                              error=function(x) ei(t,x,n, id=id,
                                      data=data, Zb=Zb, Zw=Zw,erho=3,esigma=esigma, ebeta=ebeta, ealphab=ealphab, ealphaw=ealphaw, truth=truth)), error=function(x) ei.estimate(t,x,n,
                                     id=id, data=data, Zb=Zb, Zw=Zw, erho=5,esigma=esigma, ebeta=ebeta, ealphab=ealphab, ealphaw=ealphaw, truth=truth))
    dbuf.sim <- ei.sim(dbuf)
    return(dbuf.sim)
}
  }

    if(length(dv)>1){
    print("Running eiRxC")
    #If the table is RxC use eiRxC
    dbuf <- ei.MD.bayes(formula, data=data, total=total, covariate=covariate, lambda1=lambda1, lambda2=lambda2, covariate.prior.list=covariate.prior.list, tune.list=tune.list, start.list=start.list, sample=sample, thin=thin, burnin=burnin, verbose=verbose, ret.beta=ret.beta, ret.mcmc=ret.mcmc, usrfun=usrfun)
    dbuf$data <- data
    dbuf$total <- n
    dbuf$formula <- formula
    class(dbuf) <- "ei"
    return(dbuf)
  }

}

ei.estimate <- function(t,x,n,id,Zb=1,Zw=1, data=NA, erho=.5, esigma=.5, ebeta=.5,
               ealphab=NA, ealphaw=NA, truth=NA, Rfun=2, precision=4){

#Check to make sure data is not null
  if(!missing(data)){
    t <- data[[t]]
    x <- data[[x]]
    n <- data[[n]]
    if(is.character(Zb)) Zb <- data[[Zb]]
    if(is.character(Zw)) Zw <- data[[Zw]]
    id <- data[[id]]
  }
  
  Zb <- as.matrix(Zb)
  Zw <- as.matrix(Zw)

  #If there are no covariates, run simplest R function
  if(dim(Zb)[1] == 1 & Zb[1,1] == 1 & dim(Zw)[1] == 1 & Zw[1,1] == 1) Rfun=5
  if (dim(Zb)[1]==1 & Zb[1,1]==1) Zb <- as.matrix(rep(1,length(x)))
  if (dim(Zw)[1]==1 & Zw[1,1]==1) Zw <- as.matrix(rep(1,length(x)))

  #Extract the number of covariates
  numb <- dim(Zb)[2]
  numw <- dim(Zw)[2]

  #Starting values
  start <- c(0,0,-1.2,-1.2, 0, rep(0, numb+numw))

  message("Maximizing likelihood")
 solution <- ucminf(start, like, y=t, x=x, n=n, Zb=Zb,
     Zw=Zw,numb=numb, erho=erho, esigma=esigma,
     ebeta=ebeta, ealphab =ealphab,ealphaw=ealphaw, Rfun=Rfun, hessian=3)
#This didn't work
  #solution <- optim(start, like, y=t, x=x, n=n, Zb=Zb,
  #Zw=Zw,numb=numb, erho=erho, esigma=esigma,
  #ebeta=ebeta, ealphab =ealphab, ealphaw=ealphaw, hessian=T,
  #Rfun=Rfun, method="BFGS")
  #control=list(maxeval=10))
  #print(solution$par)
  #print(solution$convergence)
  #solution <- genoud(like, y=t, x=x, n=n, Zb=Zb, Zw=Zw,numb=numb,
  #erho=erho, esigma=esigma,
  #ebeta=ebeta, ealphab
  #=ealphab, ealphaw=ealphaw, nvars=5, starting.values=start)
  #solution <- maxLik(like, y=t, x=x, n=n, Zb=Zb, Zw=Zw,numb=numb,
  #erho=erho, esigma=esigma,
  #ebeta=ebeta, ealphab =ealphab, ealphaw=ealphaw,start=start)
  #solution <- subplex(start, like, y=t, x=x, n=n, Zb=Zb,
  #Zw=Zw,numb=numb, erho=erho,esigma=esigma,
  #ebeta=ebeta, ealphab =ealphab, ealphaw=ealphaw)
  #solution <- nlminb(start, like,y=t, x=x, n=n, Zb=Zb,
  #Zw=Zw,numb=numb, erho=erho, esigma=esigma,
  #ebeta=ebeta, ealphab =ealphab, ealphaw=ealphaw, Rfun=Rfun,
  #hessian=T)
  #

  #Find values of the Hessian that are 0 or 1.
  covs <- as.logical(ifelse(diag(solution$hessian)==0|
                            diag(solution$hessian)==1,0,1))
  hessian <- solution$hessian[covs,covs]
  output <- list(solution$par, solution$hessian,hessian, erho, esigma,
                 ebeta, ealphab, ealphaw, numb, x, t, n, Zb, Zw,
                 truth,precision, covs, Rfun, id)

  names(output) <- c("phi", "hessian","hessianC",  "erho",
                     "esigma", "ebeta", "ealphab", "ealphaw", "numb",
                     "x", "t", "n", "Zb", "Zw",
                     "truth", "precision", "covs", "Rfun", "id")
  class(output) <- "ei"
  return(output)
}

 ei.sim <- function(ei.object){
   hessian <- ei.object$hessianC
   erho <- ei.object$erho
   esigma <- ei.object$esigma
   ebeta <- ei.object$ebeta
   ealphab <- ei.object$ealphab
   ealphaw <- ei.object$ealphaw
   numb <- ei.object$numb
   covs <- ei.object$covs
   Rfun <- ei.object$Rfun
   x <- ei.object$x
   t <- ei.object$t
   n <- ei.object$n
   Zb <- ei.object$Zb
   Zw <- ei.object$Zw
   truth <- ei.object$truth
   id <- ei.object$id
   precision <- ei.object$precision
  #Begin Importance Sampling
  message("Importance Sampling..")
  keep <- matrix(data=NA, ncol=(length(ei.object$phi)))
  resamp <- 0
  while(dim(keep)[1] < 100){
    keep <- .samp(t,x,n, Zb, Zw, ei.object$phi, hessian, 100, keep,
                 numb=numb, covs, erho, esigma,
                 ebeta, ealphab, ealphaw, Rfun)
    resamp = resamp + 1
  }

  #Extract values from importance sampling
  keep <- keep[2:100,]
  mu <- keep[,1:2]
  sd <- keep[,3:4]
  rho <- keep[,5]
  Bb0v <- keep[,6:(5+numb)]
  Bw0v <- keep[,(6+numb):length(ei.object$phi)]
  sd[,1] <- exp(sd[,1])
  sd[,2] <- exp(sd[,2])

  #Reparamterize
  Zb <- as.matrix(Zb)
  Zw <- as.matrix(Zw)
  Bb0v <- as.matrix(Bb0v)
  Bw0v <- as.matrix(Bw0v)
  mu1 <- mu[,1]*(.25 + sd[,1]^2)+ .5 + t(as.matrix(apply(Zb,2,
    function (x) x - mean(x)))%*%t(Bb0v))
  mu2 <- mu[,2]*(.25 + sd[,2]^2) + .5 + t(as.matrix(apply(Zw,2,
    function (x) x - mean(x)))%*%t(Bw0v))

  #phin <- dmvnorm(psi, par, log=T)
  rho <- (exp(2*rho)-1)/(exp(2*rho) +1)
  psi <- cbind(mu1, mu2, sd, rho)
  bb <- psi[,1:length(x)]
  bw <- psi[,(length(x)+1):(length(x)*2)]
  sb <- psi[,(length(x)*2+1)]
  sw <- psi[,(length(x)*2+2)]
  rho <- psi[,(length(x)*2+3)]
  omx <- 1-x
  sbw <- rho*sb*sw
  betab <- matrix(nrow=length(x),ncol=dim(keep)[1])
  betaw <- matrix(nrow=length(x),ncol=dim(keep)[1])
  homoindx <- ifelse(x==0, 1, 0)
  homoindx <- ifelse(x==1, 2, homoindx)
  enumtol=.0001
  cT0 <- t < enumtol & homoindx==0
  cT1 <- t > (1-enumtol) & homoindx==0
  ok <- ifelse(homoindx == 0 & cT0 == 0 & cT1 == 0,T, F)
  wh <- homoindx==1
  bl <- homoindx==2
  for (i in 1:dim(keep)[1]){
    sig2 <- sb[i]^2*x^2 + sw[i]^2*omx^2 + sbw[i]*2*x*omx
    omega <- sb[i]^2*x + sbw[i]*omx
    eps <- t - (bb[i,])*x - (bw[i,])*omx
    mbb <- bb[i,] + omega/sig2*eps
    vbb <- sb[i]^2 - (omega^2)/sig2
	vbb = ifelse(vbb<1*10^-32, .0001, vbb)
    s <- ifelse(vbb>=0 & vbb!=Inf & !is.na(vbb),sqrt(vbb),NaN)
    bounds <- bounds1(x,t,n)
    out<- NULL
    for(j in 1:length(x[ok])){
      out[ok][j] <- rtnorm(1, mean=mbb[ok][j], sd=s[ok][j],
                           lower=bounds[ok,][j,1],
                           upper=bounds[ok,][j,2])
    }
    out[wh] <- NA
    out[bl] <- t[bl]
    out[cT1] <- bounds[cT1,1]
    out[cT0] <- bounds[cT0,1]
    betab[,i] = out
  }
  omx <- 1 - x
  for (j in 1:length(x[ok])){
    betabs <- betab[ok,][j,]
    betaw[ok,][j,] <- t[ok][j]/omx[ok][j]-betabs*x[ok][j]/omx[ok][j]
  }

  if(sum(wh)>0){
    betaw[wh,] <- as.matrix(rep(1,dim(keep)[1]))%*%t(as.matrix(t[wh]))
  }

  if(sum(bl)>0){
    betaw[bl,] <- NA
    #betaw[bl,] <- as.matrix(rep(1,dim(keep)[1]))%*%t(as.matrix(t[bl]))
  }
  if(sum(cT1)>0){
    betaw[cT1,] <-
      as.matrix(rep(1,dim(keep)[1]))%*%t(as.matrix(bounds[cT1,3]))
  }
  if(sum(cT0)>0){
    betaw[cT0,]<-
      as.matrix(rep(1,dim(keep)[1]))%*%t(as.matrix(bounds[cT0,3]))
  }

  mbetab <- apply(betab,1,mean)
  mbetaw <- apply(betaw,1,mean)
  sdbetab <- apply(betab,1,sd)
  sdbetaw <- apply(betaw,1,sd)
  output <- list(ei.object$phi, ei.object$hessian, hessian, psi, mbetab, mbetaw,
                 sdbetab, sdbetaw, betab, betaw,resamp, erho, esigma,
                 ebeta, ealphab, ealphaw, numb, x, t, n, Zb, Zw,
                 truth,
                 precision, id)

  names(output) <- c("phi", "hessian", "hessianC","psi", "betab", "betaw",
                     "sbetab",
                     "sbetaw", "betabs", "betaws", "resamp", "erho",
                     "esigma", "ebeta", "ealphab", "ealphaw", "numb",
                     "x", "t", "n", "Zb", "Zw",
                     "truth", "precision", "id")
  class(output) <- "ei"
  return(output)
}

#prints summary function
#@x summary object


print.summary <- function(x, ...){

sum.object <- x
dec <- sum.object$precision
sum.object = sum.object[1:length(sum.object)-1]
top <- list()
num = 1
for (key in names(sum.object)){
  val <- sum.object[[key]]
  if ((is.character(val) || is.numeric(val)) && length(val) < 2){
     top[[num]] <- sum.object[[key]]
     names(top[[num]]) = key
     num=num+1
   }
}
if("Resamp" %in% names(sum.object)){
message(cat(names(top[[1]]), "=", top[[1]], ",",names(top[[2]]),
 "=", top[[2]],",",names(top[[3]]), "=", top[[3]],",",
 names(top[[4]]), "=", top[[4]],",",names(top[[5]]), "=", top[[5]]))
}

if(!("Resamp" %in% names(sum.object))){

message(cat(names(top[[1]]), "=", top[[1]], ",",names(top[[2]]),
 "=", top[[2]],",",names(top[[3]]), "=", top[[3]],",",
 names(top[[4]]), "=", top[[4]]))

}
for (key in names(sum.object)){
  val <- sum.object[[key]]
  if (!((is.character(val) || is.numeric(val)) && length(val) < 2)){
     message()
     message(key)
     print(floor(val * 10^dec)/(10^dec))
   }
}
}

#Plot to help visualize multiple dimensions.

eiRxCplot <- function(ei.object, random =FALSE, groups, groupnames = groups, informative=FALSE, threshold=.65, title="EI RxC Plot",xaxis="betab", yaxis="betaw", prop=.15, data, estimates=TRUE, legendpos = c(.69,1)){
  #ok <- !is.na(ei.object$betab)&!is.na(ei.object$betaw)
  percent = prop
  x <- ei.object$x
  t <- ei.object$t
  n <- ei.object$n
  bounds <- bounds1(x,t,n)
  bbounds <- cbind(bounds[,1], bounds[,2])
  wbounds <- cbind(bounds[,4], bounds[,3])
  bounds3 <- na.omit(bounds)
  unan <- sum(bounds3[,1]==bounds3[,2] & bounds3[,1]==bounds3[,3] & bounds3[,1]==bounds3[,4] & bounds3[,1]==1)
  n <- dim(bounds)[1]
 
  plot(c(100,200), xlim=c(0,1), ylim=c(0,1),
       col="white", xaxs="i",
       yaxs="i", main=title, xlab=xaxis, ylab=yaxis)
ok2 <- NULL
 rand <- rep(TRUE, length(x))
  if (random ==TRUE) {
       rand <- sample(c(TRUE, FALSE),length(x),replace=T, prob=c(percent,1-percent))
}

groupind <- matrix(nrow=n, ncol=length(groups))
for(i in 1:length(groups)){
 groupind[,i] <- data[,c(groups[i])]>threshold 
}

none <- apply(groupind, 1, function (x) ifelse(sum(x)==0,TRUE,FALSE))

#if (black == TRUE) {
#    bl <- data$black.ei >threshold}
#  if (hispanic==TRUE){
# his <- data$hisp.ei > threshold}
#  if (white==TRUE){
# whit <- data$whit.ei > threshold
#}
# none <- !bl & !his & !whit
#print(sum(ok2))
#if (informative==TRUE) {
#     ok2 <- bounds[,2]-bounds[,1] <.15 | bounds[,4]-bounds[,3] < .15
#}
cols = rainbow(length(groups))
 for (j in 1:length(groups)){
  for(i in 1:n){
    if (groupind[i,j] == TRUE & rand[i]==TRUE)  lines(bbounds[i,], wbounds[i,], col=cols[j], lwd=1)
  }
}

for(i in 1:n){
  if (none[i] == TRUE & rand[i]==TRUE)  lines(bbounds[i,], wbounds[i,], col="black", lwd=1)
  }

if (estimates==TRUE){
  ok <- !is.na(ei.object$betab)&!is.na(ei.object$betaw)
  betabs <- ei.object$betabs[ok,]
  betaws <- ei.object$betaws[ok,]
  betabcd <- apply(betabs,1,function(x) quantile(x, probs=c(.1,.9)))
  betabm <- apply(betabs,1, mean)
  betawcd <- apply(betaws,1,function (x) quantile(x,probs=c(.1,.9)))
  betawm <- apply(betaws,1, mean)
  #n <- dim(betabcd)[2]
  for(i in 1:sum(ok)){
    if (rand[i]==TRUE) {lines(betabcd[,i], sort(betawcd[,i],decreasing=T), col="yellow",lwd=1.5)
    	}
    if(random==FALSE){
    points(betabm, betawm, col="yellow", cex=1, pch=16)
    }
  }
}
  #if (random==TRUE) text(.85,.97,paste("Unanimous",unan))
  legend(legendpos[1], legendpos[2],c(groupnames, "Mixed", "CI of Estimates"), col=c(cols, "black", "yellow"), lwd=1)
}

#@x - ei.object to plot
#Plots in ei.

plot.ei <- function(x, ...){
  ei.object <- x
  lci.function.list <- list("tomogD" = .tomog, "tomog"=.tomogl)    # Fix for passing lci value
  function.list <- list("tomogCI"=.tomog80CI, "tomogCI95"=.tomog95CI,
                        "tomogE"=.tomogE, "tomogP" = .tomogP2,
                        "betab"=.betabd,
                        "betaw"=.betawd, "xt"=.xt, "xtc"=.xtc,
                        "xtfit"=.xtfit, "xtfitg"=.xtfitg,
                        "estsims"=.estsims, "boundXb"=.boundXb,
                        "boundXw"=.boundXw,
                        "truth"=.truthfn,"eiRxCtomog"=.bndplot,
                        "movieD"=.movieD, "movie"=.movie)
  arguments <- list(...)
  # Fix for passing lci value
  if ("lci" %in% names(arguments)){
    lci<-arguments$lci
    arguments$lci<-NULL
  } else {
    lci<-TRUE
  }

  results <- list()
  if (length(arguments)!=1) {row = ceiling(length(arguments)/2)
                             par(mfrow=c(row, 2))
                           }
  for (arg in arguments) {
    if (arg %in% names(function.list)){
      results[[arg]] <- function.list[[arg]] (ei.object=ei.object)
    } else if (arg %in% names(lci.function.list)) {
      results[[arg]] <- lci.function.list[[arg]] (ei.object=ei.object, lci=lci)    # Fix for passing lci value
    } else
      results[[arg]] <- NA
  }
}

#Calculates likelihood
#@numb Number of covariates on bb
#@Rfun -- what R function to use to calculate
#@See documentation for description of other parameters

like <- function(param, y, x, n, Zb, Zw, numb, erho, esigma, ebeta,
                 ealphab, ealphaw, Rfun){

                                        #Transform parameters
  Bb0 <- param[1]
  Bw0 <- param[2]
  sb0 <- param[3]
  sw0 <- param[4]
  rho0 <- param[5]
  Bb0v <- param[6:(5+numb)]
  Bw0v <- param[(numb+6):length(param)]
  sb=exp(sb0)
  sw=exp(sw0)
  Zb <- as.matrix(Zb)
  Zw <- as.matrix(Zw)
  bb=Bb0*(.25+sb^2) + .5 +
    as.matrix(apply(Zb,2, function(x) x - mean(x)))%*%as.matrix	(Bb0v)
  bw=Bw0*(.25+sw^2) + .5 +
    as.matrix(apply(Zw,2, function(x) x - mean(x)))%*%as.matrix(Bw0v)
  rho=(exp(2*rho0)-1)/(exp(2*rho0) +1)
  sigb2 <- sb^2
  sigw2 <- sw^2
  sigbw = rho*sb*sw

#print(c(mean(Bb0),mean(Bw0),sb0,sw0,rho0,mean(Bb0v),mean(Bw0v)))
#print(c(mean(bb),mean(bw),sb,sw,rho))

                                        #Create Demographic Categories
  homoindx <- ifelse(x==0, 1, 0)
  homoindx <- ifelse(x==1, 2, homoindx)
  enumtol=.0001
  cT0 <- y<enumtol & homoindx==0
  cT1 <- y>(1-enumtol) & homoindx==0
  ok <- ifelse(homoindx==0 & cT0==0 & cT1==0,T, F)

#Compute likelihood for different categories

#0<T<1, 0<X<1
  omx <- 1-x
  mu = bb*x + bw*omx
  epsilon = y - mu
  s2 = sigb2*(x^2) + sigw2*(omx^2) + 2*sigbw*x*omx
  omega = sigb2*x + sigbw*omx
  ebb = bb + (omega/s2)*epsilon
  vbb = sigb2 - (omega^2)/s2
  vbb = ifelse(vbb<1*10^-32, .0001, vbb)
  bounds <- bounds1(x, y, n)
  s <- ifelse(vbb>=0 & vbb!=Inf & !is.na(vbb),sqrt(vbb),NaN)
  res <- NULL
  b.s = (bounds[ok,][,2]-ebb[ok])/s[ok]
  as = (bounds[ok,][,1]-ebb[ok])/s[ok]
  res[ok] <- log(pnorm(as, lower.tail=F) - pnorm(b.s, lower.tail=F))
#res[ok] <- ifelse(abs(res[ok])==Inf, log(1*10^-15),res[ok])
#res[ok] <- ifelse(abs(res[ok])==Inf, NaN,res[ok])
#res[ok] <- log(pnorm(bounds[ok,2], mean=ebb[ok], sd=s[ok]) -
##pnorm(bounds[ok,1],#mean=ebb[ok], sd=s[ok]))s
#res[ok] <- ifelse(res[ok]==NA,-999,res[ok])
#print(summary(res))
  R <- NULL
  bs <- as.matrix(cbind(bb, bw))
  R[ok] <- .createR(ok,Rfun,bb,bw,sb,sw,rho,x)

#print(summary(R))
	
#lliki  <- -.5*(log(s2[ok])+epsilon[ok]^2/s2[ok]) + res[ok] - R[ok]
  llik.het <- -.5*sum((log(s2[ok])+(epsilon[ok]^2)/(s2[ok])))
  llik.het <- llik.het + sum(res[ok]) - sum(R[ok])

#Homogenous precincts

#X=0
  wh <- homoindx==1
  llik.wh=0
  if (sum(wh)>0){
    epsilon= y[wh]-bw[wh]
    llik.whi = -.5*(log(sigw2)+(epsilon^2)/(sigw2))
    llik.wh=-.5*sum((log(sigw2)+(epsilon^2)/(sigw2)))
    bnds=cbind(rep(0,sum(wh)),rep(1,sum(wh)))
    Ebb = bb[wh]+rho*(sb/sw)*epsilon
    vbb = sigb2*(1-rho^2)
    vbb = ifelse(vbb<1*10^-32, .0001, vbb)
    s <- ifelse(vbb>=0 & vbb!=Inf & !is.na(vbb),sqrt(vbb),NaN)
    b.s = (bnds[,2]-Ebb)/s
    as = (bnds[,1]-Ebb)/s
    res <- log(pnorm(as, lower.tail=F) - pnorm(b.s, lower.tail=F))
    #res <- log(pnorm(bnds[,2], mean=Ebb, sd=s) - pnorm(bnds[,1],
    #mean=Ebb, sd=s))
    R[wh] <- .createR(wh,Rfun,bb,bw,sb,sw,rho,x)
    llik.wh = llik.wh + sum(res)-sum(R[wh])
  }

#X=1
  bl <- homoindx==2
  llik.bl=0
  if (sum(bl)>0){
    epsilon = y[bl] - bb[bl]
    llik.bl = -.5*sum((log(sigb2)+(epsilon^2)/(sigb2)))
    bnds = cbind(rep(0, sum(bl)),rep(1,sum(bl)))
    Ebb=bw[bl] + rho*(sw/sb)*epsilon
    vbb=sigw2*(1-rho^2)
	vbb = ifelse(vbb<1*10^-32, .0001, vbb)
    s <- ifelse(vbb>=0 & vbb!=Inf & !is.na(vbb),sqrt(vbb),NaN)
    b.s = (bnds[,2]-Ebb)/s
    as = (bnds[,1]-Ebb)/s
    res <- log(pnorm(as, lower.tail=F) - pnorm(b.s, lower.tail=F))
    #res <- log(pnorm(bnds[,2], mean=Ebb, sd=s) - pnorm(bnds[,1],
    #mean=Ebb, sd=s)) #res[ok] <- ifelse(abs(res[ok])==Inf,
    #NaN,res[ok])
    R[bl] <- .createR(bl,Rfun,bb,bw,sb,sw,rho,x)
    llik.bl = llik.bl + sum(res)-sum(R[bl])
  }

#T=0, 0<X<1
  llik.cT0=0
  if(sum(cT0)>0){
    bb.cT0 = bs[cT0,][1]
    bw.cT0 = bs[cT0,][2]
    sigma = matrix(c(sigb2,sigbw,sigbw,sigw2),nrow=2)
    if(sum(cT0)==1){
      first = log(dmvnorm(c(0,0),mean=bs[cT0,], sigma=sigma))
      second <- .createR(cT0,Rfun,bb,bw,sb,sw,rho,x)
      llik.cT0=sum(first)-sum(second)
    }
    else{
      first = apply(bs[cT0,], 1,
        function (x) log(dmvnorm(c(0,0),mean=as.vector(x),
                                 sigma=sigma)))
      second <- NULL
      second <- .createR(cT0,Rfun,bb,bw,sb,sw,rho,x)
      llik.cT0=sum(first)-sum(second)
    }
  }

#T=1, 0<X<1
  llik.cT1=0
  if(sum(cT1)>0){
    bb.cT1 = bs[cT1,][1]
    bw.cT1 = bs[cT1,][2]
    sigma=matrix(c(sigb2,sigbw,sigbw,sigw2),nrow=2)
    if(sum(cT1)==1){
      first = log(dmvnorm(c(1,1),mean=bs[cT1,], sigma=sigma))
    #qi <- pmvnorm(lower=c(-bb.cT1/sb,-bw.cT1/sw),
     #upper=c(-bb.cT1/sb + 1/sb, -bw.cT1/sw + 1/sw),
      #mean=c(0,0), #corr=matrix(c(1,rho,rho,1), nrow=2))
    #qi <- ifelse(qi<0 | qi==0, 1*10^-322,qi)
    #second = ifelse(qi<0|qi==0, -999,log(qi))
      second <- .createR(cT1,Rfun,bb,bw,sb,sw,rho,x)
      llik.cT1=sum(first)-sum(second)
    }
    if(sum(cT1)>1){
      first = apply(as.matrix(bs[cT1,]), 1,
        function (x) log(dmvnorm(c(1,1),mean=as.vector(x),
                                 sigma=sigma)))
      second <- NULL
      second <- .createR(cT1,Rfun,bb,bw,sb,sw,rho,x)
#for (i in 1:length(bb.cT1)){
#qi <- pmvnorm(lower=c(-bb.cT1[i]/sb,-bw.cT1[i]/sw), upper=c(-bb.cT1[i]/sb + 1/sb, -bw.cT1[i]/sw + 1/sw), mean=c#(0,0), corr=matrix(c(1,rho,rho,1), nrow=2))
#qi <- ifelse(qi<0 | qi==0, 1*10^-322,qi)
#second[i] = ifelse(qi<0|qi==0, -999,log(qi))
#}
      llik.cT1=sum(first)-sum(second)
    }
  }
  llik=llik.het + llik.bl + llik.wh + llik.cT0 + llik.cT1


#Priors
  prior=0
  lpdfnorm = log(dnorm(rho0,0,sd=erho))
  if (esigma>0) prior = prior-(1/(2*esigma^2))*(sigb2+sigw2);
  if(erho>0) prior = prior +lpdfnorm;
  if(ebeta>0 & (mean(bb)<0)) prior = prior -.5*((mean(bb)^2)/ebeta);
  if(ebeta>0 & mean(bb)>1) prior = prior -.5*((mean(bb)-1)^2/ebeta);
  if(ebeta>0 & mean(bw)<0) prior = prior -.5*((mean(bw)^2)/ebeta);
  if(ebeta>0 & mean(bw)>1) prior = prior -.5*((mean(bw)-1)^2/ebeta);
  if(sum(is.na(ealphab))==0) prior=prior +
    sum(dmvnorm(Bb0v, ealphab[,1], sigma=diag(ealphab[,2]^2), log=T));
  if(sum(is.na(ealphaw))==0) prior=prior +
    sum(dmvnorm(Bw0v, ealphaw[,1], sigma=diag(ealphaw[,2]), log=T));
  llik = llik + prior
                                        #print(-llik)
  if(is.na(llik)|abs(llik)==Inf) llik = NaN
  return(-llik)
}


#Alternative res
#s = sqrt(vbb)
#b.s = (bounds[,2]-ebb)/s
#as = (bounds[,1]-ebb)/s
#log(pnorm(b.s) - pnorm(as))

#for (i in 1:75) {
#res[i] <- log(pmvnorm(lower=c(as[i]), upper=c(b.s[i]), mean=c(0), #sigma=as.matrix(1)))
#}

#for (i in 1:75) {
#res[i] <- log(pmvnorm(lower=c(bounds[,1][i]), upper=c(bounds[,2][i]), #mean=ebb[i], sigma=as.matrix(vbb[i])))
#}
#Alternative R

#R <- ifelse(rho==0,
		#log(pmvnorm(lower=c(0), upper=c(1), mean=bb, #sigma=as.matrix(sigb2))) + log(pmvnorm(lower=c(0), upper=c(1), #mean=bw, sigma=as.matrix(sigw2))), log(pmvnorm(lower=c(0,0), #upper=c(1,1), mean=c(bb, bw), sigma=matrix(c(sigb2,sigbw,sigbw,sigw2), nrow=2))))
		

#@x - vector of characteristics, e.g.  perc. of blacks in
#each district
#@t - vector o fcharacteristics, e.g. perc. of people that voted in
#each district

#Computes the bounds for each precinct
bounds1 <- function(x,t,n){
  #set basic values
  homindx <- NULL
  tx <- NULL
  tomx <- NULL
  LbetaB <- NULL
  UbetaB <- NULL
  LbetaW <- NULL
  UbetaW <- NULL
  omx = 1-x
  Nb = x*n
  Nw =omx*n
  p = length(x)
  homoindx <- ifelse(x==0, 1, 0)
  homoindx <- ifelse(x==1, 2, homoindx)


  #Heterogenous precincts
  tx <- as.matrix(t/x)
  tomx = as.matrix(t/omx)
  tomxx <- as.matrix(tx-(omx/x))
  txx <- as.matrix(tomx-x/(1-x))
  LbetaB <- apply(tomxx, 1, function (x) max(0,x))
  UbetaB <- apply(tx, 1, function (x) min(x,1))
  LbetaW <- apply(txx, 1, function (x) max(0, x))
  UbetaW <- apply(tomx, 1, function (x) min(x, 1))
  
  #Homogenously black
  bl <- homoindx==2
  LbetaB[bl] = t[bl]
  UbetaB[bl] = t[bl]
  LbetaW[bl] = NA
  UbetaW[bl] = NA
	

  #Homogenously white
  wh <- homoindx==1
  LbetaB[wh] = NA
  UbetaB[wh] = NA
  LbetaW[wh] = t[wh]
  UbetaW[wh] = t[wh]
		
return(cbind(LbetaB, UbetaB, LbetaW, UbetaW))
}
	

# 3-d Tomography Plot for 2x3 Case

tomogRxC3d <- function(formula,data,total=NULL,lci=TRUE,estimates=FALSE,ci=FALSE,level=.95,seed=1234,color=hcl(h=30,c=100,l=60),transparency=.75,light=FALSE,rotate=TRUE){
  ##Run Through RxC Code Once (from Molly's original tomogRxC function)
  #require(grDevices)

  if (!requireNamespace("rgl", quietly = TRUE)) {
    stop("Package rgl is needed for the tomogRxC3d function to work. Please install it.",
      call. = FALSE)
  }
  
  noinfocount <- 0
  form <- formula
  dvname <- terms.formula(formula)[[2]]
  covariate <- NA
  #Make the bounds
  rows <- c(all.names(form)[6:(length(all.names(form)))])
  names <- rows
  cols <- c(all.names(form)[3])
  options(warn=-1)
  bnds <- bounds(form, data=data,rows=rows, column =cols,threshold=0)
  options(warn=0)
  #Totals
  dv <- data[, all.names(form)[3]]
  #Assign other category
  bndsoth <- bnds$bounds[[3]]
  oth <- data[,all.names(form)[length(all.names(form))]]
  
  #Assign x-axis category
  bndsx <- bnds$bounds[[1]]
  xcat <- data[,all.names(form)[6]]
  
  #Assign y-axis category
  bndsy <- bnds$bounds[[2]]
  ycat <- data[,all.names(form)[7]]
  
  #Minimums & Maximums
  minx <- bndsx[,1]
  miny <- bndsy[,1]
  minoth <- bndsoth[,1]
  maxx <- bndsx[,2]
  maxy <- bndsy[,2]
  maxoth <- bndsoth[,2]
  
  #####
  #Starting point when x is at minimum
  ##
  #Holding x at its minimum, what are the bounds on y?
  
  #When x is at its minimum, the new dv and total are:
  newdv <- dv - (minx*xcat)
  newtot <- oth + ycat 
  t <- newdv/newtot
  y <- ycat/newtot
  
  #The new bounds on the y category are:
  
  lby <- cbind(miny, (t - maxoth*oth/newtot)/(y))
  lby[,2] <- ifelse(y==0, 0, lby[,2])
  lowy <- apply(lby,1,max)
  hby <- cbind((t-minoth*oth/newtot)/y,maxy)
  highy <- apply(hby,1,min)
  
  #####
  #Starting point when x is at maximum
  ##
  #Holding x at its maximum, what are the bounds on y?
  
  #The new bounds on x are:
  newtot <- oth + xcat
  newdv <- dv - (miny*ycat)
  x <- xcat/newtot
  t <- newdv/newtot
  lbx <- cbind(minx, (t-maxoth*oth/newtot)/x)
  lbx[,2] <- ifelse(x==0, 0, lbx[,2])
  lowx <- apply(lbx,1,max)
  hbx <- cbind((t-minoth*oth/newtot)/x,maxx)
  highx <- apply(hbx,1,min)
  
  #Graph starting points
  #High starting points
  hstr <- cbind(minx, highy)
  #High ending points
  hend <- cbind(highx, miny)
  
  #Low starting points
  lstr <- cbind(minx, lowy)
  lend <- cbind(lowx, miny)
  
  xl <- paste("Percent", names[1], dvname[2])
  yl <- paste("Percent", names[2], dvname[2])
  zl <- paste("Percent", names[3], dvname[2])
  mn <- paste("3-d Tomography Plot in a 2x3 Table")
  
  #plot(c(0,0), xlim=c(0,1), ylim=c(0,1), xaxs="i", yaxs="i",xlab=xl, ylab=yl, col="white", main=mn)
  
  
  ok <- !is.na(hstr[,2]) & !is.na(hend[,1])
  exp1 <- hstr[ok,2]>=maxy[ok]
  exp2 <- hend[ok,1]>=maxx[ok]
  exp3 <- lstr[ok,2]<=miny[ok]
  exp4 <- lend[ok,1]<=minx[ok]
  hstr <- hstr[ok,]
  hend <- hend[ok,]
  lstr <- lstr[ok,]
  lend <- lend[ok,]
  dv <- dv[ok]
  ycat <- ycat[ok]
  oth <- oth[ok]
  minoth <- minoth[ok]
  xcat <- xcat[ok]
  maxy <- maxy[ok]
  maxx <- maxx[ok]
  
  ##Create Matrices for coordinates
  coords.xaxs<-matrix(data=NA,nrow=dim(hstr)[1],ncol=10)
  coords.yaxs<-matrix(data=NA,nrow=dim(hstr)[1],ncol=10)
  
  for(i in 1:dim(hstr)[1]){
    if((exp1[i] + exp2[i] + exp3[i] + exp4[i])==0){
      xaxs <- c(hstr[i,1],  lstr[i,1],lend[i,1],hend[i,1])
      yaxs <- c(hstr[i,2], lstr[i,2],lend[i,2], hend[i,2])
      side1 <- c(xaxs,xaxs[1])
      side2 <- c(yaxs,yaxs[1])
      c1 <- sum(side1[1:(length(side1)-1)]*side2[2:length(side2)])
      c2 <- sum(side1[2:(length(side1))]*side2[1:(length(side2)-1)])
      area <- abs(c1-c2)/2
      #polygon(xaxs,yaxs, col=rgb(0,0,0,alpha=0), border="black", lty=1, lwd=.25)
    }
    if((exp1[i]==1) & (exp2[i])==0){
      cut <- (dv[i]-(oth[i])*minoth[i])/xcat[i] - maxy[i]*ycat[i]/xcat[i]
      kink1x <- c(cut)
      kink1y <- c(maxy[i])
    }
    if((exp2[i]==1) & (exp1[i])==0){
      cut <- (dv[i]-(oth[i])*minoth[i])/ycat[i] - maxx[i]*xcat[i]/ycat[i]
      kink1x <- c(maxx[i])
      kink1y <- c(cut)
    }
    if((exp2[i]==1 & exp1[i]==1)){
      cut <- (dv[i]-(oth[i])*minoth[i])/ycat[i] - maxx[i]*xcat[i]/ycat[i]
      cut2 <- (dv[i]-(oth[i])*minoth[i])/xcat[i] - maxy[i]*ycat[i]/xcat[i]
      kink1x <- c(maxx[i], cut2)
      kink1y <- c(cut, maxy[i])
    }
    if((exp3[i]==1) & (exp4[i])==0){
      cut <- (dv[i]-(oth[i])*maxoth[i])/xcat[i] - miny[i]*ycat[i]/xcat[i]
      kink2x <- c(cut)
      kink2y <- c(miny[i])
    }
    if((exp4[i]==1) & (exp3[i])==0){
      cut <- (dv[i]-(oth[i])*maxoth[i])/ycat[i] - minx[i]*xcat[i]/ycat[i]
      kink2x <- c(minx[i])
      kink2y <- c(cut)
    }
    if((exp3[i]==1 & exp4[i]==1)){
      cut <- (dv[i]-(oth[i])*maxoth[i])/ycat[i] - minx[i]*xcat[i]/ycat[i]
      cut2 <- (dv[i]-(oth[i])*maxoth[i])/xcat[i] - miny[i]*ycat[i]/xcat[i]
      kink2x <- c(minx[i], cut2)
      kink2y <- c(cut, miny[i])
    }
    if((exp3[i] + exp4[i])==0 & (exp1[i] + exp2[i] + exp3[i] + exp4[i])!=0){
      xaxs <- c(hstr[i,1],  lstr[i,1],lend[i,1],hend[i,1], kink1x)
      xaxs <- ifelse(is.nan(xaxs), 0, xaxs)
      yaxs <- c(hstr[i,2], lstr[i,2],lend[i,2], hend[i,2], kink1y)
      yaxs <- ifelse(is.nan(yaxs), 0, yaxs)
      side1 <- c(xaxs,xaxs[1])
      side2 <- c(yaxs,yaxs[1])
      c1 <- sum(side1[1:(length(side1)-1)]*side2[2:length(side2)])
      c2 <- sum(side1[2:(length(side1))]*side2[1:(length(side2)-1)])
      area <- abs(c1-c2)/2
      #polygon(xaxs,yaxs, col=rgb(0,0,0,alpha=0), border="black", lty=1, lwd=.25)
    }
    if((exp1[i] + exp2[i])==0 & (exp1[i] + exp2[i] + exp3[i] + exp4[i])!=0){
      xaxs <- c(hstr[i,1],  lstr[i,1],kink2x,lend[i,1],hend[i,1])
      xaxs <- ifelse(is.nan(xaxs), 0, xaxs)
      yaxs <- c(hstr[i,2], lstr[i,2],kink2y,lend[i,2], hend[i,2])
      yaxs <- ifelse(is.nan(yaxs), 0, yaxs)
      side1 <- c(xaxs,xaxs[1])
      side2 <- c(yaxs,yaxs[1])
      c1 <- sum(side1[1:(length(side1)-1)]*side2[2:length(side2)])
      c2 <- sum(side1[2:(length(side1))]*side2[1:(length(side2)-1)])
      area <- abs(c1-c2)/2
      #polygon(xaxs,yaxs, col=rgb(0,0,0,alpha=0), border="black", lty=1, lwd=.25)
    }
    if((exp1[i] + exp2[i])!=0 & (exp3[i] + exp4[i])!=0){
      xaxs <- c(hstr[i,1],  lstr[i,1],kink2x,lend[i,1],hend[i,1], kink1x)
      xaxs <- ifelse(is.nan(xaxs), 0, xaxs)
      yaxs <- c(hstr[i,2], lstr[i,2],kink2y,lend[i,2], hend[i,2], kink1y)
      yaxs <- ifelse(is.nan(yaxs), 0, yaxs)
      side1 <- c(xaxs,xaxs[1])
      side2 <- c(yaxs,yaxs[1])
      c1 <- sum(side1[1:(length(side1)-1)]*side2[2:length(side2)])
      c2 <- sum(side1[2:(length(side1))]*side2[1:(length(side2)-1)])
      area <- abs(c1-c2)/2
      #polygon(xaxs,yaxs, col=rgb(0,0,0,alpha=0), border="black", lty=1, lwd=.225)
    }
    for(j in 1:length(xaxs)){if(xaxs[j]<1e-5){xaxs[j]<-0}}
    for(k in 1:length(yaxs)){if(yaxs[k]<1e-5){yaxs[k]<-0}}
    for(l in 1:length(xaxs)){if(xaxs[l]>.99999){xaxs[l]<-1}}
    for(m in 1:length(yaxs)){if(yaxs[m]>.99999){yaxs[m]<-1}}
    axs<-rbind(xaxs,yaxs)
    coords.axs<-unique(axs,MARGIN=2,incomparables=F)
    coords.axs<-if(length(coords.axs[1,])!=10){cbind(coords.axs,matrix(data=NA,nrow=2,ncol=(10-length(coords.axs[1,]))))}else{coords.axs}
    coords.xaxs[i,]<-coords.axs[1,]
    coords.yaxs[i,]<-coords.axs[2,]
  }
  
  ##Record Coordinate
  w.coord<-coords.xaxs[,colSums(is.na(coords.xaxs))<nrow(coords.xaxs)]
  b.coord<-coords.yaxs[,colSums(is.na(coords.yaxs))<nrow(coords.yaxs)]
  
  
  #######################################################################
  
  ##THIS WILL BE A FN WHERE YOU SPECIFY NAME OF TURNOUT, TOTAL, CATEGORIES
  
  ##Combine Coordinates for 3d Graph
  w<-matrix(NA,nrow=dim(hstr)[1],ncol=dim(w.coord)[2])
  b<-matrix(NA,nrow=dim(hstr)[1],ncol=dim(w.coord)[2])
  h<-matrix(NA,nrow=dim(hstr)[1],ncol=dim(w.coord)[2])
  
  #Extract Data needed for accounting Identity
  N <- data[,all.names(form)[3]] + data[,all.names(form)[4]]
  t <- data[,all.names(form)[3]]/N
  xb <- data[,all.names(form)[7]]/N
  xw <- data[,all.names(form)[6]] /N
  
  #Using the Accounting Identity from King (1997), calculate coordinates for remaining category
  ##T = B_b*X_b + B_w*X_w + B_h*X_h = B_b*X_b + B_w*X_w + B_h*(1-X_b-X_h)
  coords<-matrix(NA, nrow=dim(w.coord)[2], ncol=3)
  for(i in 1: dim(hstr)[1]){
    coords<-cbind(w.coord[i,],b.coord[i,],rep(NA, nrow(coords)))  
    for(j in 1:nrow(coords)){
      #Account for cases where xh=0
      if(1-xb[i]-xw[i] < 1e-5){coords[j,3]<-0}else{
        coords[j,3]<-(t[i] - coords[j,2]*xb[i] - coords[j,1]*xw[i])/(1-xb[i]-xw[i])}
      ##Make Sure that it records small numbers as 0
      if(coords[j,3]<1e-5 & is.na(coords[j,3])==F){coords[j,3]<-0}
      if(coords[j,3]>.99999 & is.na(coords[j,3])==F){coords[j,3]<-1}
      ##Account for cases where xb=0 and xw
      if(xb[i]==0){coords[j,2]<-0}
      if(xw[i]==0){coords[j,1]<-0}
    }
    w[i,]<-coords[,1]
    b[i,]<-coords[,2]
    h[i,]<-coords[,3]
  }
  
  ##Run RxC Model
  set.seed(seed)
  dbuf <- ei(formula=formula, data=data)
  
  ##Calculate Center of Ellipsoid: mean of estimates
  means<-apply(dbuf$draws$Beta, MARGIN=2, FUN=mean) #Mean of all simulations for each precinct and beta
  means.w <- means[seq(1, length(means), 6)]
  means.b <- means[seq(2, length(means), 6)]
  means.h <- means[seq(3, length(means), 6)]
  
  ##Calculate Variance of Ellipsoid: average variance of simulations
  ses.w <- sd(means.w)
  ses.b <- sd(means.b)
  ses.h <- sd(means.h)
  
  ##Calculate Covariance of Ellipsoid (Rho): average covariance of simulations
  covs.bw<-cov(means.w,means.b)
  covs.wh<-cov(means.w,means.h)
  covs.bh<-cov(means.b,means.h)
  #Create Rho Matrix
  #Rho if covariance divided by multiplied variance
  rho<-matrix(c(1,covs.bw/(ses.b*ses.w),covs.bh/(ses.b*ses.h),covs.bw/(ses.b*ses.w),1,covs.wh/(ses.h*ses.w)
                ,covs.bh/(ses.b*ses.h),covs.wh/(ses.h*ses.w),1),nrow=3)
  
  
  ######################################
  
  ##CALCULATE AREA OF PLANES for LoCoI
  area <- rep(NA, nrow(w))
  for(i in 1:nrow(w)){
    #2-sided Line
    if(length(unique(w[i,]))==1 | length(unique(b[i,]))==1 | length(unique(h[i,]))==1){
      if(length(na.omit(w[i,]))>2){area[i]<-(sqrt(abs(h[i,1]-h[i,2])^2 + abs(b[i,1]-b[i,2])^2))*.01} #Multiply by epsilon to weight it appropriately small
      if(length(na.omit(b[i,]))>2){area[i]<-(sqrt(abs(h[i,1]-h[i,2])^2 + abs(w[i,1]-w[i,2])^2))*.01}
      if(length(na.omit(h[i,]))>2){area[i]<-(sqrt(abs(w[i,1]-w[i,2])^2 + abs(b[i,1]-b[i,2])^2))*.01}
    }
    # All other Planes
    if(length(unique(w[i,]))!=1 & length(unique(b[i,]))!=1 & length(unique(h[i,]))!=1){
      # Get 3d coordinates
      p <- cbind(b[i,],w[i,],h[i,])
      p <- na.omit(p)
      
      # Get equation of plane
      fit <- lm(h[i,] ~ b[i,] + w[i,])
      coefs <- coef(fit)
      a.eq <- coefs[2]
      b.eq <- coefs[3]
      c.eq <- -1
      d.eq <- coefs["(Intercept)"]
      
      # Get normal vector
      n <- matrix(c(a.eq,b.eq,c.eq), nrow=1, ncol=3)
      
      # Calculate Basis Vector on Plane
      u1 <- matrix(c( (p[2,1]-p[1,1]), (p[2,2]-p[1,2]), (p[2,3]-p[1,3])), nrow=1, ncol=3)
      
      # Get Second Basis Vector as cross-product of n and u1
      CrossProduct3D <- function(x, y, i=1:3) {
        # Project inputs into 3D, since the cross product only makes sense in 3D.
        To3D <- function(x) head(c(x, rep(0, 3)), 3)
        x <- To3D(x)
        y <- To3D(y)
        # Indices should be treated cyclically (i.e., index 4 is "really" index 1, and
        # so on).  Index3D() lets us do that using R's convention of 1-based (rather
        # than 0-based) arrays.
        Index3D <- function(i) (i - 1) %% 3 + 1
        # The i'th component of the cross product is:
        # (x[i + 1] * y[i + 2]) - (x[i + 2] * y[i + 1])
        # as long as we treat the indices cyclically.
        return (x[Index3D(i + 1)] * y[Index3D(i + 2)] -
                  x[Index3D(i + 2)] * y[Index3D(i + 1)])
      }
      
      u2 <- matrix(CrossProduct3D(n,u1), nrow=1, ncol=3)
      
      # Normalize the basis vectors
      e1 <- u1/sqrt(u1[1,1]^2 + u1[1,2]^2 + u1[1,3]^2)
      e2 <- u2/sqrt(u2[1,1]^2 + u2[1,2]^2 + u2[1,3]^2)
      
      # Get the projection matrix from basis vectors
      pm <- rbind(e1, e2)
      
      # Get the 2d coordinates from projection matrix
      p.2d <- t(pm%*%t(p))
      
      # Create matrix of points in counter-clockwise order (need to systematize)
      ccorder <- rev(chull(p.2d)) #Convex hull gives clockwise order then reverse
      parea <- p.2d[order(ccorder),]
      
      # Add first row to bottom of matrix
      parea <- rbind(parea, parea[1,])
      
      # Calcluate area as 1/2 of determinant (need to systematize)
      plus.area <- 0
      for(j in 2:dim(parea)[1]){
        x <- parea[j-1,1]*parea[j,2]
        plus.area <- plus.area + x
      }
      minus.area <- 0
      for(k in 2:dim(parea)[1]){
        x <- parea[k-1,2]*parea[k,1]
        minus.area <- minus.area + x
      }
      area[i] <- 1/2*(plus.area-minus.area)
    }
  }
  
  scale<-(area-min(area))/(max(area)-min(area))*100
  
  #################################
  ## Plot Based on Plot Type
  
  ##Tomography Plot - with/without point estimates
  if(ci==F){
    rgl::plot3d(b,w,h, type="s", col="black", size=0.3, xlim=c(0,1), ylim=c(0,1), zlim=c(0,1), xlab=yl, ylab=xl, zlab=zl)
    ##Add the plane for each set of points
    for(i in 1:dim(hstr)[1]){  
      #Two point plane (line)
      if(length(unique(w[i,]))==1 | length(unique(b[i,]))==1 | length(unique(h[i,]))==1){
        if(lci==T){rgl::lines3d(x = b[i,], y = w[i,], z=h[i,], col=hcl(h=30,c=100,l=scale[i],alpha=1))}
        else{rgl::lines3d(x = b[i,], y = w[i,], z=h[i,], col=color)}
      }else{
        #All Others
        fit <- lm(h[i,] ~ b[i,] + w[i,])
        coefs <- coef(fit)
        a <- coefs[2]
        c <- coefs[3]
        d <- -1
        e <- coefs["(Intercept)"]
        if(lci==T){rgl::planes3d(a, c, d, e, alpha=transparency, col=hcl(h=30,c=100,l=scale[i]), lit=light)}
        else{rgl::planes3d(a, c, d, e, alpha=transparency, col=color, lit=light)}
      }
    }
    # With point estimates - mean of average simulations across precincts
    if(estimates==T){rgl::spheres3d(means.b,means.w, means.h, col="black", radius=.03, add=T)}
  }
  
  ##With Confidence Elipse
  if(ci==T){  
    rgl::plot3d(rgl::ellipse3d(x=rho , scale= c(ses.b,ses.w,ses.h), centre= c(mean(means.b),mean(means.w), mean(means.h))
                     , alpha=.8, level=level) ,color=hcl(h=10,c=60,l=20), xlim=c(0,1), ylim=c(0,1), zlim=c(0,1), xlab=yl, ylab=xl, zlab=zl, lit=light)
    rgl::points3d(mean(means.b),mean(means.w), mean(means.h), col="black", size=10, add=T) #Mean of average simulations across precincts
    rgl::points3d(b,w,h, col="black", size=1, add=T)
    ##Add the plane for each set of points
    for(i in 1:dim(hstr)[1]){  
      #Two point plane (line)
      if(length(unique(w[i,]))==1 | length(unique(b[i,]))==1 | length(unique(h[i,]))==1){
        if(lci==T){rgl::lines3d(x = b[i,], y = w[i,], z=h[i,], col=hcl(h=30,c=100,l=scale[i],alpha=1))}
        else{rgl::lines3d(x = b[i,], y = w[i,], z=h[i,], col=hcl(h=30,c=100,l=60,alpha=1))}
      }else{
        #All Others
        fit <- lm(h[i,] ~ b[i,] + w[i,])
        coefs <- coef(fit)
        a <- coefs[2]
        c <- coefs[3]
        d <- -1
        e <- coefs["(Intercept)"]
        if(lci==T){rgl::planes3d(a, c, d, e, alpha=.25, col=hcl(h=30,c=100,l=scale[i]), lit=light)}
        else{rgl::planes3d(a, c, d, e, alpha=.25, col=hcl(h=30,c=100,l=60), lit=light)}
      }
    }
    # With point estimates - mean of average simulations across precincts
    if(estimates==T){rgl::spheres3d(means.b,means.w, means.h, col="black", radius=.03, add=T)}
  }
  
  if(rotate==T){rgl::play3d( rgl::spin3d(rpm=2.5), duration=20)}
}

#@ei.object -- ei object
#@... options
#Extracts information from ei object

eiread <- function(ei.object, ...){
  function.list <- list("betab" = .betaB, "betaw" = .betaW,
                        "phi" = .phi, "sbetab" = .sbetab,
                        "sbetaw" = .sbetaw, "psisims" = .psisims,
                        "bounds" = .bounds, "CI80b" =.CI80b,
                        "CI80w" = .CI80w, "abounds" = .abounds,
                        "aggs" = .aggs, "maggs" = .maggs,
                        "VCaggs" = .VCaggs, "eaggbias" = .eaggbias,
                        "goodman" = .goodman)
  dec <- ei.object$precision
  arguments <- list(...)
  results <- list()
  for (arg in arguments) {
    if (arg %in% names(function.list))
      results[[arg]] <-
        floor(function.list[[arg]](ei.object)*10^dec)/10^dec
    else
      results[[arg]] <- NA
  }
  if (length(results)==1)
    results <- results[[1]]
  if (length(results) <1)
    warning("qi results object is empty")
  #class(results) <- "eiread"
  results
}

%\VignetteIndexEntry{EI: A(n R) Program for Ecological Inference}
%\VignetteDepends{eiPack}



\SweaveOpts{results=hide,prefix.string=vigpics/EI}
%\include{zinput}
\documentclass[11pt,oneside,letterpaper,titlepage]{article}
%\usepackage[notref]{showkeys} % [notref]
\usepackage{Rd/Rd}
\usepackage{Sweave}
\usepackage{upquote}

\usepackage[reqno]{amsmath}
%\usepackage{natbib}
\usepackage{graphicx
}
\usepackage{graphics}
\usepackage{amssymb}
\usepackage{amsthm}
\usepackage{stmaryrd}
\usepackage{bbm}
\usepackage{epsf}
%\usepackage{psfig}
%\usepackage{wcolor}
\usepackage{textcomp} % for \texttrademark
\usepackage{makeidx}

\usepackage{verbatim}
% \usepackage{epsf}
\usepackage{url}
%\usepackage{html}
\usepackage{dcolumn}
\usepackage{longtable}
%\usepackage{vmargin}
%\topmargin=1in
% === margin and formatting ===
\usepackage{setspace}
\usepackage{fullpage}
\usepackage{caption}

\def\fp#1{\mathbbm{#1}}

%\setpapersize{USletter}
\newcolumntype{.}{D{.}{.}{-1}}
\newcolumntype{d}[1]{D{.}{.}{#1}}
%\pagestyle{myheadings}  %commented out by JH
%\htmladdtonavigation{
%  \htmladdnormallink{%
%    \htmladdimg{http://gking.harvard.edu/pics/home.gif}}
%  {http://gking.harvard.edu/}}
%\newcommand{\hlink}{\htmladdnormallink}
%
%\bodytext{ BACKGROUND="http://gking.harvard.edu/pics/temple.jpg"}
\setcounter{tocdepth}{3}



%\VignetteIndexEntry{Non-parametric estimation of Ecological Inference}
%\VignetteDepends{ei,mvtnorm}
%\VignetteKeyWords{model,non-parametric, ecological}
%\VignettePackage{ei}

\title{EI: A(n R) Program for Ecological Inference} \author{Gary King
  \thanks{Albert J. Weatherhead III University Professor, Harvard
    University (Institute for Quantitative Social Sciences, 1737
    Cambridge Street, Harvard University, Cambridge MA 02138;
    http://GKing.Harvard.Edu, King@Harvard.Edu, 617-500-7570.} \and
  Margaret Roberts \thanks{Ph.D.\ Candidate, Department of Government,
    1737 Cambridge Street, Harvard University, Cambrdige MA 02138}}

\begin{document}

\maketitle
\tableofcontents

<<beforepkgs, echo=FALSE>>=
 before<-search()
 ind <- grep("[a-z A-Z]+:ei",before)
 if(!length(ind) > 0) library(ei)
@
\include{ei.toc}

\section[Introduction: Ecological Inference]{{\tt Introduction}: Ecological Inference}
\label{intro}

This program provides methods of inferring individual behavior from
aggregate data. It implements the statistical procedures, diagnostics,
and graphics from the Gary King, \textbf{A Solution to the Ecological
  Inference Problem: Reconstructing Individual Behavior from Aggregate
  Data} (Princeton: Princeton University Press, 1997),
\url{http://j.mp/ecinfbook}; Ori Rosen, Wenxin Jiang, Gary King, and
Martin A..\ Tanner (2001) ``Bayesian and Frequentist Inference for
Ecological Inferece: The RxC Case.''  (\emph{Statistica Neerlandica},
Vol.55, nr.2, pp 134-156), \url{http://j.mp/eiRxC}; and from our own
ongoing work.  Except where indicated, all references to page,
section, chapter,
table, and figure numbers in this document refer to the the first reference.\\

\noindent Ecological inference is the process of using aggregate
(i.e., ``ecological'') data to infer discrete individual-level
relationships of interest when individual-level data are not
available.  Ecological inferences are required in political science
research when individual-level surveys are unavailable (e.g., local or
comparative electoral politics), unreliable (racial politics),
insufficient (political geography), or infeasible (political history).
They are also required in public policy (e.g., for applying the Voting
Rights Act) and other academic disciplines ranging from epidemiology
and marketing to sociology and quantitative history. Most researchers
using aggregate data have encountered some form of the ecological inference problem.\\

\noindent Because the ecological inference problem is caused by the
lack of individual-level information, no method of ecological
inference, including that estimated by this program, will always
produce accurate results.  However, potential difficulties are reduced
by models that include more available information, diagnostics to
evaluate when assumptions need to be modified, easy methods of
modifying the assumptions, and uncertainty estimates for quantities of
interest.  We recommend reviewing Chapter 16 while using this program
for actual research.  Model dependence defines ecological inference
when going anywhere beyond the bounds and tomography plots, and so we
have added new graphical procedures for evaluating the degree of model dependence. \\

\noindent \verb#eiR# is a unifed program that contains methods for
both the 2x2 ecological inference case and the RxC case.  We will
begin by explaining how to use eiR in the 2x2 case, and then move on
to RxC examples.

\section[Overview: 2x2 R-commands]{{\tt Overview}: 2x2 R-commands}
\label{commands}

In this section we describe the basic commands for 2x2 inference.  For
this purpose, and without loss of generality, we use the running
example from the book portrayed in Table 2.3 (page 31).  This example
uses the proportion of the voting age population who are black
($X_i$), the proportion turning out to vote ($ T_i$), and the number
of voting age people ($ {N_i}$) in each precinct ( $ i=1,\ldots,p$) to
infer the fraction of blacks who vote ($ \beta_i^b$) and the
proportion of whites who vote ($ \beta_i^w$), also in each precinct.
(For an extended example of \verb#eiR#, refer to the user's guide
below.)  An example of a sequence of these commands are as follows:

<<echo=TRUE, eval=FALSE>>=
formula = t ~ x
dbuf <- ei(formula=formula, total="n",data=data)
summary(dbuf)
eiread(dbuf, "betab","betaw")
plot(dbuf, "tomog","betab","betaw","xtfit")
@

\noindent In most applications, \verb#plot# and \verb#eiread# 
would likely be run multiple times with different options chosen, and other commands would 
be included with these five to read in the data (\verb#t#, \verb#x#, and \verb#n#).\\
\newline
 
\begin{itemize}
\item{{\verb#ei#}: To run the main procedure}

  Use the format, \verb#dbuf = ei(formula,total="n",data=data)#, which takes three $
  p\times 1$ vectors as inputs: \verb#t# (e.g. the fraction of the
  voting age population turning out to vote); \verb#x# (e.g. the
  fraction of the voting age population who are black); and \verb#n#
  (e.g. the total number of people in the voting age population). The output of this
  procedure is the list \verb#dbuf# (i.e. an output object of class
  list called a ``data buffer'').  The data buffer includes the maximium likelihood estimates and simulated quantities of interest from the \verb#ei# procedure. After running \verb#ei#, it is a
  good idea to save \verb#dbuf# for further analysis.  The
  output data buffer from \verb#ei# includes a large variety of
  different results useful for understanding the results of the
  analysis. A minimal set of nonrepetitive information is stored in
  this list (or data buffer), and a large variety of other information
  can be easily computed from it. Fortunately, you do not need to know
  whether the information
  you request is stored or computed as both are treated the same.\\
  \newline To extract information from the data buffer, three
  procedures are available:

\item{{\verb#summary#}: To obtain a general summary of district-level information}

  \verb#summary(dbuf)# will produce aggregate information about the
  \verb#ei# results.  \verb#summary# will produce a summary of both the maximum likelihood estimates and simulated quantities of interest. It also includes information on the values specified for the prior.

\item{{\verb#plot#}: To graph relevant information}

  For graphics, use \verb#plot(dbuf, "name")#;, where \verb#dbuf# is
  the ei.object that is the output of \verb#ei# or \verb#ei.sim#, and name can be any number
  of a long list of ready-made graphs.  For example,
  \verb#plot(dbuf,"tomog")# will plot a tomography graph, and
  \verb#plot(dbuf,"xt")# will display a scattercross graph. Any number
  of graphs can be selected in plot for output.  For example,
  \verb#plot(dbuf,"tomog","xt")# will produce both a tomography graph
  and a scattercross graph.

  Some of the plots can be produced after estimating the maximum likelihood estimates using \verb#ei#, but before simulating quantities of interest. These initial plots can be useful in determining which priors to include, and the user can opt to choose not to simulate quantities of interest by selecting the option \verb#simulate=FALSE#. However, for some plots, \verb#plot# will need the simulated quantities of interest, and therefore can only be plotted when simulate is set to the default, \verb#TRUE#.
  
\item{{\verb#eiread#}: To obtain relevant information and numerical results}

  For numerical information, use \verb#v <- eiread(dbuf, "name")#,
  where \verb#v# is the item extracted, \verb#dbuf# is the data buffer
  output from \verb#ei#, and name can be any of a list of output
  possibilities. For example, \verb#eiread(dbuf, "betab")# will give a
  vector of point estimates of $ \beta_i^b$,
  \verb#eiread(dbuf, "ci80w")# will give 80\% confidence intervals for
  $ \beta_i^w$.  Any number of items can be selected for output.  For
  example, \verb#eiread(dbuf, "betab", "ci80b")# will produce a list
  of the point estimates and 80\% confidence intervals for $
  \beta_i^b$.

\end{itemize}

\section{User's Guide: An Example}

We now show how to use \verb#eiR# through a simple example. We use data
on voter registration and racial background of people from 268
counties in four Southern U.S. States: Florida, Louisiana, and South
Carolina.  These are the same data used in Chapter 10. The data
include the total voting age population in each county (\verb#n#), the
proportion of the population in each county who are black (\verb#x#),
and the proportion of the population in each county who are registered
to vote (\verb#t#).  The proportion of whites registered can be
computed by (\verb#1-x#). You can load the data into R using the
command

<<echo=TRUE>>=
library(ei)
data(matproii)
@

\noindent The statistical goal is to estimate the fraction of blacks registered ($\beta_i^b$) and the fraction of whites registered ($\beta_i^w$) in each county. These quantities of interest are generally unknown.  However, \verb#eiR# also includes an option that allows the user to assess the reliability of the \verb#eiR# algorithm when the true quantities of interest are known. To illustrate the use of this ``truth" option, the data also include the true fractions of registered blacks (\verb#tb#) and the true fraction of registered whites (\verb#tw#). 

\subsection{The Basic EI Algorithm}

To begin, we perform ecological inference by calling the function
\verb#ei#. \verb#ei# first computes and maximizes a likelihood
distribution based on the inputed data, then estimates the county-level
quantities of interest based on the possible values, or ``bounds", for
each county and the overall likelihood distribution.  \\ \newline To
run this algorithm, at a minimum we need to have three vectors.  Two
vectors, \verb#t# and \verb#x#, contain the aggregate known
information about the counties of interest.  In this example, \verb#t#
is the proportion of voters registered in each county, and \verb#x# is
the proportion of blacks in each county.  These two vectors contain
aggregate information, since we are interested in the proportion of
voters in each country who are black.  The last vector we need is
\verb#n#, the number of people of interest in each county.  In this
example, \verb#n# is the number of people of voting age.\\ \newline We
proceed by performing ecological inference without covariates on the
data:

%<<echo=TRUE>>=
<<echo=TRUE, eval=FALSE>>=
formula = t ~ x
dbuf = ei(formula=formula,total="n",data=matproii)
@

\noindent To include a covariate on $\beta_i^b$ simply specify a covariate vector for the option \verb#Zb#, or a string that refers to the name of the covariate in the dataset.  Similarly, to include a covariate on $\beta_i^w$, specify a covariate vector for for the option \verb#Zw#, or a string that refers to the name of the covariate in the dataset.\\
\newline
Next, we use \verb#summary(dbuf)#and \verb#summary(dbufsim)# to obtain general information about the estimation. 

<<echo=TRUE, eval=FALSE>>=
summary(dbuf)
@

\begin{verbatim}
Erho = 0.5 , Esigma = 0.5 , Ebeta = 0.5 , N = 268 , Resamp = 35

Maximum likelihood results in scale of estimation (and se's)
    Bb0    Bw0    sigB    sigW    rho Zb Zw
 1.2670 1.9348 -1.1151 -1.3272 1.6051  0  0
 0.2775 0.2757  0.2132  0.1640 0.3120  0  0

Untruncated psi's
     BB     BW     SB     SW    RHO
 0.9758 1.1411 0.3325 0.2713 0.9082

Truncated psi's (ultimate scale)
     BB     BW     SB     SW    RHO
 0.6208 0.8259 0.2006 0.1425 0.7489

Aggregate Bounds
       betab  betaw
lower 0.2125 0.7025
upper 0.9754 0.9200

Estimates of Aggregate Quantities of Interest
     mean     sd
Bb 0.5714 0.0253
Bw 0.8177 0.0072

\end{verbatim}

\noindent The \verb#summary# function provides basic information about the ecological inference estimation, the maximum likelihood estimates on three scales, and an overall summary of the quantities of interest.  First, it reports the values of the priors used in ecological inference (\verb#Erho, Esigma, Ebeta#).  It also reports the number of counties in the dataset (\verb#N#), as well as the number of importance sampling iterations required to produce estimates of the quantities of interest (\verb#resamp#).  \\
\newline
\verb#summary# also produces information both about the maximum likelihood estimates and the quantities of interest.  First, it provides the maximum likelihood estimates in the scale of estimation. Next, it provides the values of the MLEs when they are transformed into an untruncated bivariate normal distribution.  These estimates provide information about the location of the highest density of estimates for the proportion of blacks who are registered to vote and the proportion of whites who are registered to vote.  Last, it provides the values of the MLEs on the ultimate truncated bivariate normal scale.  These estimates are an unweighted average of the fraction of blacks and whites registered to vote over all the counties in the sample.  In this example, the \verb#ei# algorithm predicts that the unweighted average of the proportion of blacks registered to vote across counties is 0.57 and the unweighted average of the proportion of whites registered to vote is 0.82.\\
\newline
Finally, \verb#summary# produces information on aggregrate quantities of interest.  The aggregate bounds are the mean of the bounds on the proportions of black and white voters, weighted by the population in each county.  The aggregate quantities of interest are the weighted mean of the proportion of registered blacks and the proportion of registered whites in each county.  In this example the weighted average proportion of blacks who are registered is 0.57, and the weighted average proportion of whites who are registered is 0.82.  

\subsection{Extracting Quantities of Interest}

\verb#eiread# extracts quantities of interest in a list format from the object \verb#dbuf#.  For example, 

<<echo=TRUE, eval=FALSE>>=
bb.out <- eiread(dbuf, "betab", "sbetab")
@

\noindent extracts the point estimates and estimated standard deviations for $\beta_i^b$, the estimates of the proportion of registered blacks in each county.  The user can then use \verb#bb.out$betab# to obtain a vector of the point estimates for $\beta_i^b$, and \verb#bb.out$sbetab# to obtain a vector of the standard devations for $\beta_i^b$.  \\
\newline
\verb#eiread()# takes any number of arguments to extract any number of quantities of interest from the data buffer.  \verb#?eiread# can be used to find a list of quantities of interest that are available in \verb#EI#.  Among the most useful arguments are \verb#"betab"# and \verb#"betaw"#, which report the point estimates; \verb#"sbetab"# and \verb#"sbetaw"#, which report the standard deviations of the point estimates; and \verb#"CI80b# and \verb#"CI80w"#, which report the confidence intervals of the point estimates.  

\subsection{Plotting in EI}

Plotting \verb#EI# output is extremely useful for understanding the results of the ecological inference algorithm and diagnosing problems with estimation.  First, we graph a tomography plot of the data (Figure 1).

<<echo=TRUE,eval=FALSE>>=
plot(dbuf, "tomog")
@

\begin{figure}[h]
\begin{center}
<<results=tex,echo=FALSE, eval=FALSE>>= 
png("tomogplot.png", width=6, height=6, pointsize=10)
plot(dbuf, "tomog")
invisible(dev.off())
@
\includegraphics[width=3in, height=3in,   viewport=0 0 420 395,clip]{tomogplot.png} 
\end{center}
\caption{Tomography Plot with ML Contours}
\label{fig:tomog}
\end{figure}


\begin{figure}[h]
\begin{center}
<<results=tex,echo=FALSE, eval=FALSE>>= 
png("tomogplot2.png", width=14, height=6, pointsize=10)
plot(dbuf, "tomogE", "tomogCI")
invisible(dev.off())
@
\includegraphics[width=7.5in, height=2.5in,   viewport=0 0 1400 395,clip]{tomogplot2.png} 
\end{center}
\caption{Tomography Plots: Point Estimates (left) and Confidence
  Intervals (right)}
\label{fig:tomog2}
\end{figure}

\noindent Each line on the map represents the possible values for $\beta_i^b$ and $\beta_i^w$ for one county.  The contour lines identify the portion of the lines that have the highest probability of containing the true estimates of $\beta_i^b$ and $\beta_i^w$.  These contour lines provide information about the overall pattern of registration.  Note that the area with highest probability is in the upper right-hand corner, where the proportion of whites registered is between $0.75$ and $1$ and the proportion of blacks registered is between $0.5$ and $1$.  Further, we see that the lines are clustered in the upper half of the figure, indicating that the possible values of $\beta_i^w$ will have a lower variance than the possible values of $\beta_i^b$.\\
\newline
Figure 2 is a double plot of the point estimates and their confidence intervals.  To compute this, we call plot to generate two graphs: a tomography plot with the point estimates generated from the algorithm, and a tomography plot with 80\% confidence intervals on the point estimate.  

<<echo=TRUE,eval=FALSE>>=
plot(dbuf, "tomogE","tomogCI")
@

\noindent This plot is useful to visualize the estimates and confidence intervals for each county.  We can see that the point estimates and confidence intervals are clustered in the same area as the contours from the previous plot.  Further, the point estimates and confidence intervals only fall on the lines that indicate the possible values of $\beta_i^b$ and $\beta_i^w$.\\
\newline
Figure 3 shows plots that indicate the distribution of $\beta_i^b$ and $\beta_i^w$. To produce these plots, run:

<<echo=TRUE,eval=FALSE>>=
plot(dbuf, "betab","betaw")
@

\noindent Density plots are useful to visualize the location and uncertainty of the point estimates.  The green line represents the density of the simulated point estimates, and the black tick marks are a rug plot of the point estimates,s $\beta_i^b$ and $\beta_i^w$.  You can see that the variance of the point estimates from $\beta_i^b$ is much higher than the variance of point estimates from $\beta_i^w$. \\ 
\newline
Figure 4 portrays the results of the \verb#EI# algorithm by plotting the proportion of blacks in each country by the proportion of registered voters in each county.  To produce this plot, use:

<<echo=TRUE,eval=FALSE>>=
plot(dbuf, "xtfit")
@

\noindent The circles around each of the points in this plot are proportional to the population of each county.  The graph represents the likelihood estimates by plotting the expected number of registered voters given the proportion of blacks in each county, represented by the yellow line. The red lines in the plot are the 80\% confidence interval around the regression line.  The higher uncertainty in the estimates of black registration can be seen by the absence of points on the right hand side of the graph and the larger confidence interval on the right hand side of the graph, where the proportion of blacks in the county is relatively large.  


\begin{figure}[h]
\begin{center}
<<results=tex,echo=FALSE, eval=FALSE>>= 
png("betas.png", width=14, height=6, pointsize=10)
plot(dbuf, "betab", "betaw")
invisible(dev.off())
@
\includegraphics[width=8.5in, height=2.5in,   viewport=0 0 1400 395,clip]{betas.png} 
\end{center}
\caption{Plots with Distribution of Point Estimates for $\beta_i^b$
  (left) and $\beta_i^w$ (right), for all $i$}
\label{fig:betas}
\end{figure}


\begin{figure}[h]
\begin{center}
<<results=tex,echo=FALSE, eval=FALSE>>= 
png("xtfit.png", width=6, height=6, pointsize=10)
plot(dbuf, "xtfit")
invisible(dev.off())
@
\includegraphics[width=2.75in, height=2.75in, viewport=0 0 420 400,clip]{xtfit.png} 
\end{center}
\caption{X and T Scatterplot with E(T|X) and 80\% CIs}
\label{fig:xtfit}
\end{figure}

\pagebreak
\noindent Finally, if we have data on the true proportions of registered blacks and registered whites in each county, as we do in this dataset, we can use plots in \verb#EI# to assess how well the algorithm works on the given data.  To do this, rerun the \verb#ei# algorithm, adding the truth vectors, \verb#tb# and \verb#tw# as the truth argument.

<<echo=TRUE, eval=FALSE>>=
truth = cbind(matproii$tb,matproii$tw)
formula = t ~ x
dbuf = ei(formula=formula,total="n",data=matproii,truth=truth)
@

\noindent Then use plot to compare the estimates of the \verb#EI# algorithm to the true proportions of white and black registered voters in each county.

<<echo=TRUE, eval=FALSE>>=
plot(dbuf, "truth")
@

\noindent The ``truth" plot (Figure 5) has four components.  The top two figures have the posterior densities of the aggregate quantities of interest $B^b$ and $B^w$.  These densities tell us in what range the algorithm predicts that the point estimates lie.  For example, the density of $B^b$ is wider than the density of $B^w$, indicating that our estimates of $B^w$ lie in a smaller range than than our estimates of $B^b$.  The true aggregate values, computed from the \verb#truth# data we inputed, are indicated by the vertical bars.  The fact that the true $B^b$ and $B^w$ are within the densities we computed confirms that the model is did a good job at predicting the true proportions of registered white and black voters.\\
\newline
\noindent The bottom two figures in Figure 5 plot the estimated values of $\beta_i^b$ and $\beta_i^w$ against the true values.  The size of each of the circles plotted is proportional to the number of blacks in the county in the first graph and the number of whites in each county in the second graph. If the estimated values were exactly equal to the true values, all of the points would be on the $45^{\circ}$ line.  Because the points fall quite close to the $45^{\circ}$ and do not deviate from the line in a systematic way, we can see that the \verb#EI# algorithm predicts the point estimates quite well.

\begin{figure}[ht]
\begin{center}
<<results=tex,echo=FALSE, eval=FALSE>>= 
png("truth2.png", width=6, height=6, pointsize=10)
plot(dbuf, "truth")
invisible(dev.off())
@
\includegraphics[width=2.75in, height=2.75in, viewport=0 0 420 395,clip]{truth2.png} 
\end{center}
\caption{Comparing Estimates to the Truth at the County Level}
\label{fig:truth}
\end{figure}

\clearpage

\section[Overview: eiRxC R-commands]{{\tt Overview}:eiRxC R-commands}

Although the 2x2 case is useful in many circumstances, eiR also allows users to expand beyond the 2x2 case and implement RxC models.  To do this, eiR contains the MCMC implementation of the multinomial-Dirichlet eiRxC model in Rosen, Jiang, King and Tanner (2001) ``Bayesian and Frequentist Inference for Ecological Inferece: The RxC Case." (\emph{Statistica Neerlandica}, Vol. 55, nr.2, pp 134-156), and incorporates R code from \verb#eiPack# (Olivia Lau, Ryan Moore and Mike Kellermann). \\

\noindent In this section we describe the basic commands needed for eiRxC.  Imagine that instead of a 2x2 table where the user aims to infer the proportion of whites and non-whites who turned out to vote, for example, the user instead wants to infer the percent of turnout for white, black, and hispanic.  This is a 2x3 table because the resulting estimates from this table will include two estimates (percent turnout and no turnout) for three categories of people (white, black and hispanic). \\

\noindent We use the data \verb#RxCdata# as a running example.
\verb#RxCdata# has variables for each of the five marginal variables
for the 2x3 table: \verb#turnout# is the number of people in each precinct who turned out to vote, \verb#noturnout# is the number of people in each precinct that did not turn out to vote, \verb#white# is the number of people who are white in the precinct, \verb#black# is the number of people who are black in the precinct, and \verb#hisp# is the number of people who are hispanic in the precinct. Note that, different that the 2x2 case, these are the numbers of people rather than proportions.  The user can include proportions if they want to, but then must include the argument \verb#total# for the the total number of people in the precinct. \\

\noindent To begin, we load the RxC data from the \verb#eiR# library.
<<echo=TRUE, eval=FALSE>>=
data(RxCdata)
@

The formula for the eiRxC case tells the program which variables go on each side of the table.  In this example, we have turnout and no turnout on one side of the table, and white, black and hispanic on the other side.  Therefore, our formula is:

<<echo=TRUE, eval=FALSE>>=
formula <- cbind(turnout, noturnout) ~ cbind(white, black, hisp)
@

We can call this formula using ei:
<<echo=TRUE, eval=FALSE>>=
dbuf = ei(formula, data=RxCdata)
@

The MCMC output for the $\beta$'s can be found in the data buffer \verb#dbuf#.  \verb#dbuf$draws$Beta# is a matrix that contains the simulations for each of the $\beta$'s for each precinct.  Since there are 60 precincts in this dataset and 6 quantities of interest, the number of columns of \verb#dbuf$draws$Beta# is $60*6 = 360$.  We can analyze the MCMC output by picking out which $\beta$ we are interested at any given time.  For example, to look at the MCMC output for the estimated percent white turnout for the third precinct we can use the R code:

<<echo=TRUE, eval=FALSE>>=
out <- dbuf$draws$Beta[,"beta.white.turnout.3"]
summary(out)
@

\begin{verbatim}
Iterations = 1:1000
Thinning interval = 1 
Number of chains = 1 
Sample size per chain = 1000 

1. Empirical mean and standard deviation for each variable,
   plus standard error of the mean:

          Mean             SD       Naive SE Time-series SE 
      0.430233       0.035612       0.001126       0.003312 

2. Quantiles for each variable:

  2.5%    25%    50%    75%  97.5% 
0.3615 0.4078 0.4306 0.4552 0.5007 

\end{verbatim}

We can also plot this MCMC output to assess convergence:
<<echo=TRUE, eval=FALSE>>=
plot(out)
@

\begin{figure}[ht]
\begin{center}
\includegraphics{MCMCplot.png} 
\end{center}
\caption{MCMC Output for \% White Turnout in Precinct 3}
\end{figure}

\noindent From this output we can get the empirical mean and standard
deviation of each of our quantities of interest.  The user can add in
extra arguments to specify the number of simulations, burnin, or
priors for this function.  For more information on this, see the
documentation for \verb#eiPack# and the Rosen et al (2001) paper.

\clearpage

\section{eiRxC Tomography Plots: Assessing Model Dependence}

While eiRxC model estimates are helpful, the uncertainty estimates
they produce are --- like all ecological inference point estimates ---
conditional on the model assumed. Yet, most of the actual scientific
uncertainty in ei models does not stem from the sampling uncertainty
given a statistical model, but rather the uncertainty associated with
choosing a model --- what is known as model dependence (King and Zeng 2006). A tomography plot that contains information about
the possible values of the quantities of interest is therefore more
useful than the model in understanding the true amount of information
in the data from which the model is obtaining its estimates.\\ 

\noindent While this has not been available in the RxC case before, we
have developed a method to creating and displaying tomography plots
for 2x3 RxC problems and by extension examine model dependence in any
RxC model.\\

\noindent In the 2x3 case, we can do this by using the function
\verb#tomogRxC#. Following the code above, using the \verb#RxCdata#,

<<echo=TRUE, eval=FALSE>>=
formula <- cbind(turnout, noturnout) ~ cbind(white, black, hisp)
tomogRxC(formula, RxCdata)
@ 

\begin{figure}[ht]
\begin{center}
\includegraphics{whiteblackRxC.png} 
\end{center}
\caption{2x3 Tomography Plot: \% White Turnout and \% Black Turnout}
\end{figure}

\noindent Each of the polygons on the plot below represent all the
possibile values in a given precinct for percent white turnout and
percent black turnout. The plot follows what we call the Law of
Conservation of Ink, where tomography polygons with more information
(smaller polygons) are colored brighter than those with less
information (larger polygons).  The colors on the plot are a heat map
that are brighter when polygons overlap each other. The polygons in
this particular district contain a lot of information about the
percent white turnout, as the vast majority of the polygons only
contain low values of percent white turnout, but they contain very
little information about the percent black turnout, as most of the
polygons seem to cover the entire range possible for percent black
turnout.

\clearpage

\noindent The quantity of interest percent hispanic turnout is left
out of this tomography plot, since we can only graph two quantities of
interest simulataneously.  However, if we reorganize the formula,
putting white and hispanic first on the right hand side of the
formula, we can create another tomography plot that expores the
deterministic information for percent white turnout and percent
hispanic turnout.

<<echo=TRUE, eval=FALSE>>=
formula <- cbind(turnout, noturnout) ~ cbind(white, hisp, black)
tomogRxC(formula, RxCdata)
@ 

\begin{figure}[ht]
\begin{center}
\includegraphics{whitehispRxC.png} 
\end{center}
\caption{2x3 Tomography Plot: \% White Turnout and \% Hispanic Turnout}
\end{figure}


\noindent With this new tomography plot (in Figure 8), we find very similar results to the previous one.  While we have quite a bit of information about the percent white turnout, we have less information about percent hispanic turnout.  This result derives from the fact that there are very few hispanics and blacks in this particular district.

\clearpage
Last, we can plot the remaining two quantities of interest -- percent black turnout and percent hispanic turnout-- against each other:

<<echo=TRUE, eval=FALSE>>=
formula <- cbind(turnout, noturnout) ~ cbind(hisp, black, white)
tomogRxC(formula, RxCdata)
@ 

\begin{figure}[ht]
\begin{center}
\includegraphics{hispblackRxC.png} 
\end{center}
\caption{2x3 Tomography Plot: \% Hispanic Turnout and \% Black Turnout}
\end{figure}


Here we see that even though the informative polygons indicate low values of percent hispanic and black turnout, we have the least information about the intersection of percent black and hispanic turnout. The polygons are spread all across the unit square space, indicating that the possible values for percent black and hispanic turnout spread over a large area.  In the bottom right hand corner, and on the R console, the tomography plot reports how many of the tomography polygons have no information, or how many tomography polygons cover the entire unit square.  Here we see that ther are 34 tomography polygons with no information, and since we onlyhave 60 precincts, this means that we have very little information for these quantities of interest. We might be more confident in the estimates for white turnout in this RxC analysis than for black and hispanic turnout.

\clearpage
\section{Reference to EI Functions}
\include{Rd/ei}
\include{Rd/eiread}
\include{Rd/plot.ei}
\include{Rd/tomogRxC}

\bibliography{References}
\label{bib}
\begin{thebibliography}{6}

\bibitem{gking}Gary King, \emph{A Solution to the Ecological Inference Problem: 
Reconstructing Individual Behaviour from Aggregate Data}, Princeton University Press (1997).

\bibitem{KinZen06} Gary King and Langche Zeng, ``The Dangers of Extreme Counterfactuals.'' \emph{Political Analysis}, 14:2, pp 131-159, 2006.
\bibitem{lauetal} Lau et al, ``eiPack: Ecological Inference and Higher-Dimension Dtaa Management." \url{http://www.olivialau.org/software}.

\bibitem{rosenetal} Rosen et al, ``Bayesian and Frequentist Inference for Ecological Inference: The RxC Case." \emph{Statistica Neerlandica}, 55:2, pp 134-156, 2001.

\bibitem{harvardweb}Useful documentation also avalaible at \url{http://GKing.Harvard.Edu}.

\bibitem{Rweb}For R language see \url{http://www.r-project.org}.

\bibitem{statR}Venables, W.N.,and Ripley, B.D., \emph{Statistics and Computing}, Springer (2002).

\end{thebibliography}



\end{document}

     \name{census1910}
     \docType{data}
     \alias{census1910}
     \title{Black Literacy in 1910}
     \description{
     	A dataset of aggregate literacy rates (t) and fraction of the population that is black (x), from the 1910 US Census.  Each observation represents one county.
     }
     \usage{census1910}
     \format{A data frame containing 1030 observations.}
     \source{Gary King, 1997, "Replication data for: A Solution to the Ecological Inference Problem: Reconstructing Individual Behavior from Aggregate Data", http://hdl.handle.net/1902.1/LWMMKUTYXS UNF:3:DRWozWd89+vNLO7lY2AHbg== IQSS Dataverse Network [Distributor] V3 [Version]}
     \references{
     Gary King. (1997). \emph{A Solution to the Ecological Inference Problem.}  Princeton: Princeton University Press.  Section 13.2:241-5.
     
     Robinson, William S.  (1950).  ``Ecological Correlation and the Behavior of Individuals.''  \emph{American Sociological Review} 15:351-357.
     }
     \keyword{datasets}

\name{tomogRxC}
\alias{tomogRxC}

\title{
  Plotting Ecological Inference Estimates with eiRxC information
}
\description{
	A tomography plot for an estimated Ecological Inference model in RxC data.
}
\usage{
tomogRxC(formula, data, total=NULL, refine=100)
}
\arguments{
  \item{formula}{A formula of the form \code{cbind(col1, col2,...)~cbind(row1,row2,...)}}
 \item{data}{data that contains the data that corresponds to the
formula}
\item{total}{`total' is the name of the variable in the dataset that
	  contains the number of individuals in each unit}
\item{refine}{specifies the amount of refinement for the image.  Higher
numbers mean better resolution.} 
}
 
\references{
Gary King (1997). A Solution to the Ecological Inference Problem.  Princeton: Princeton University Press.
}
\author{
  Gary King <<email: king@harvard.edu>> and Molly Roberts <<email: molly.e.roberts@gmail.com>>
}

\examples{
data(RxCdata)
formula = cbind(turnout, noturnout) ~ cbind(white, black,hisp)
tomogRxC(formula, data=RxCdata)
}

     \name{fultongen}
     \docType{data}
     \alias{fultongen}
     \title{Voter Transitions}
     \description{
     	Aggregated data from 289 precincts in Fulton County, Georgia.  The variable \code{t} represents the fraction voting in 1994 and \code{x} the fraction in 1992.  Beta_b is then the fraction who vote in both elections, and Beta_w the fraction of nonvoters in 1992 who vote in the midterm election of 1994.
     }
     \usage{fultongen}
     \format{A data frame containing 289 observations.}
     \source{Gary King, 1997, "Replication data for: A Solution to the Ecological Inference Problem: Reconstructing Individual Behavior from Aggregate Data", http://hdl.handle.net/1902.1/LWMMKUTYXS UNF:3:DRWozWd89+vNLO7lY2AHbg== IQSS Dataverse Network [Distributor] V3 [Version]}
     \references{
     Gary King (1997). \emph{A Solution to the Ecological Inference Problem.}  Princeton: Princeton University Press.  Section 13.1:235-41.
     }
     \keyword{datasets}

\name{summary.ei}
\alias{summary.ei}

\title{
Summarize Ecological Inference Estimates
}
\description{
`summary' method for the class `ei'.
}
\usage{
\method{summary}{ei}(object, \dots)
}

\arguments{
  \item{object}{An \code{ei} object from the function \code{ei}.}
\item{\dots}{A list of options to return in graphs.  See values below.} 
}
 
\author{
  Gary King <<email: king@harvard.edu>> and Molly Roberts <<email: molly.e.roberts@gmail.com>>
}
\references{
Gary King (1997). A Solution to the Ecological Inference Problem.  Princeton: Princeton University Press.
}
\examples{
data(sample)
formula = t ~ x
dbuf <- ei(formula=formula, total="n",data=sample)
print(summary(dbuf))
}

\name{ei}
\alias{EI}
\alias{ei}
\title{
Ecological Inference Estimation
}
\description{
\code{ei} is the main command in the package \code{EI}.  It gives observation-level estimates (and various related statistics) of \eqn{\beta_i^b} and \eqn{\beta_i^w} given variables \eqn{T_i} and \eqn{X_i} (\eqn{i=1,...,n}) in this accounting identity: \eqn{T_i=\beta_i^b*X_i + \beta_i^w*(1-X_i)}.  Results are stored in an \code{ei} object, that can be read with \code{summary()} or \code{eiread()} and graphed in \code{plot()}.
}
\usage{
ei(formula, total, id, data, Zb=1, Zw=1,data=NA, erho=.5, esigma=.5, ebeta=.5, ealphab=NA,
 ealphaw=NA, truth=NA, precision=4)
}

\arguments{
	\item{formula}{A formula of the form \eqn{t ~x} in the \eqn{2x2}
	  case and \eqn{cbind(col1,col2,...) ~ cbind(row1,row2,...)} in
	  the RxC case.}
	\item{n}{`total' is the name of the variable in the dataset that
	  contains the number of individuals in each unit}
	\item{id}{`id' is the nae of the variable in the dataset that
	  identifies the precinct. Used for `movie' and `movieD' plot functions.}
	\item{Zb}{
\eqn{p} x \eqn{k^b} matrix of covariates or the name of covariates in
the dataset}
	\item{Zw}{
\eqn{p} x \eqn{k^w} matrix of covariates or the name of covariates in
the dataset}
	\item{data}{optional data frame that if specified should contain \code{t},\code{x},\code{n}.  If using covariates and data is specified, data should also contain \code{Zb} and \code{Zw}.}
	\item{erho}{The standard deviation of the normal prior on \eqn{\phi_5} for the correlation. Default \eqn{=0.5}.}
	\item{esigma}{The standard deviation of an underlying normal distribution, from which a half normal is constructed as a prior for both \eqn{\breve{\sigma}_b} and \eqn{\breve{\sigma}_w}. Default \eqn{= 0.5}}
	\item{ebeta}{Standard deviation of the "flat normal" prior on \eqn{\breve{B}^b} and \eqn{\breve{B}^w}.  The flat normal prior is uniform within the unit square and dropping outside the square according to the normal distribution.  Set to zero for no prior. Setting to positive values probabilistically keeps the estimated mode within the unit square. Default\eqn{=0.5}}
	\item{ealphab}{cols(Zb) x 2 matrix of means (in the first column) and standard deviations (in the second) of an independent normal prior distribution on elements of \eqn{\alpha^b}.  If you specify Zb, you should probably specify a prior, at least with mean zero and some variance (default is no prior).  (See Equation 9.2, page 170, to interpret \eqn{\alpha^b}).}
	\item{ealphaw}{cols(Zw) x 2 matrix of means (in the first column) and standard deviations (in the second) of an independent normal prior distribution on elements of \eqn{\alpha^w}.  If you specify Zw, you should probably specify a prior, at least with mean zero and some variance (default is no prior).  (See Equation 9.2, page 170, to interpret \eqn{\alpha^w}).}
	\item{truth}{A length(t) x 2 matrix of the true values of the quantities of interest.}
	\item{precision}{The number of decimal places that are printed
	  in \code{summary} and \code{eiread} functions.}
	\item{...}{see documentation in \code{eiPack} for options for
	  RxC ei.} 
}
\details{
	The \code{EI} algorithm is run using the \code{ei} command.  A summary of the results can be seen graphically using \code{plot(ei.object)} or numerically using \code{summary(ei.object)}.  Quantities of interest can be calculated using \code{eiread(ei.object)}.
}

\author{
  Gary King <<email: king@harvard.edu>> and Molly Roberts <<email: roberts8@fas.harvard.edu>>
}
\references{
Gary King (1997). A Solution to the Ecological Inference Problem.  Princeton: Princeton University Press.
}

\examples{
data(sample)
form <- t ~ x
dbuf <- ei(form,total="n",data=sample)
summary(dbuf)
}

\name{eiread}
\alias{eiread}
\title{
Quantities of Interest from Ecological Inference Estimation
}
\description{
\code{eiread} is the command that pulls quantities of interest from the \code{ei} object.  The command returns a list of quantities of interest requested by the user.
}
\usage{
eiread(ei.object, ...)
}

\arguments{
	\item{ei.object}{An \code{ei} object from the function \code{ei}.}
	\item{\dots}{A list of quantities of interest for \code{eiread()} to return.  See values below.} 
	}

\value{
\item{betab}{\eqn{p} x \eqn{1} point estimate of \eqn{\beta_i^b} based on its mean posterior.  See section 8.2}
	\item{betaw}{\eqn{p} x \eqn{1} point estimate of \eqn{\beta_i^w} based on its mean posterior.  See section 8.2}
	\item{sbetab}{\eqn{p} x \eqn{1} standard error for the estimate of \eqn{\beta_i^b}, based on the standard deviation of its posterior.  See section 8.2}
	\item{sbetaw}{\eqn{p} x \eqn{1} standard error for the estimate of \eqn{\beta_i^w}, based on the standard deviation of its posterior.  See section 8.2}
	\item{phi}{Maximum posterior estimates of the CML}
	\item{psisims}{Matrix of random simulations of \eqn{\psi}.  See section 8.2}
	\item{bounds}{\eqn{p} x \eqn{4}: bounds on \eqn{\beta_i^b} and \eqn{\beta_i^w}, lowerB ~ upperB ~ lowerW ~ upperW.  See Chapter 5.}
	\item{abounds}{\eqn{2} x \eqn{2}: aggregate bounds rows:lower, upper; columns: betab, betaw.  See Chapter 5.}
	\item{aggs}{Simulations of district-level quantities of interest \eqn{\hat{B^b}} and \eqn{\hat{B^w}}.  See Section 8.3.}
	\item{maggs}{Point estimate of 2 district-level parameters, \eqn{\hat{B^b}} and \eqn{\hat{B^w}} based on the mean of aggs.  See Section 8.3.}
	\item{VCaggs}{Variance matrix of 2 district-level parameters, \eqn{\hat{B^b}} and \eqn{\hat{B^w}}.  See Section 8.3.}
	\item{CI80b}{\eqn{p} x \eqn{2}: lower~upper \eqn{80\%}
 confidence intervals for \eqn{\beta_i^b}.  See section 8.2.}
	\item{CI80w}{\eqn{p} x \eqn{2}: lower~upper \eqn{80\%} confidence intervals for \eqn{\beta_i^w}.  See section 8.2.}
	\item{eaggbias}{Regressions of estimated \eqn{\beta_i^b} and \eqn{\beta_i^w} on a constant term and \eqn{X_i}.}
	\item{goodman}{Goodman's Regression.  See Section 3.1}
}
\author{
  Gary King <<email: king@harvard.edu>> and Molly Roberts <<email: molly.e.roberts@gmail.com>>
}
\references{
Gary King (1997). A Solution to the Ecological Inference Problem.  Princeton: Princeton University Press.
}

\examples{
data(sample)
formula = t ~ x
dbuf <- ei(formula=formula, total="n",data=sample)
eiread(dbuf, "phi")
eiread(dbuf, "betab", "betaw")
}

     \name{lavoteall}
     \docType{data}
     \alias{lavoteall}
     \title{Turnout by Race in Louisiana}
     \description{
     	The fraction of blacks registered voters (x) and fraction of voter turnout (t) in each Louisiana precinct, along with the true fraction of black turnout (tb) and non-black turnout (tw).
     }
     \usage{lavoteall}
     \format{A data frame containing 3262 observations.}
     \source{Gary King, 1997, "Replication data for: A Solution to the Ecological Inference Problem: Reconstructing Individual Behavior from Aggregate Data", http://hdl.handle.net/1902.1/LWMMKUTYXS UNF:3:DRWozWd89+vNLO7lY2AHbg== IQSS Dataverse Network [Distributor] V3 [Version]}
     \references{
     Gary King (1997). \emph{A Solution to the Ecological Inference Problem.}  Princeton: Princeton University Press.  Section 1.4:22-4.
     }
     \keyword{datasets}

\name{bounds1}
\alias{bounds1}

\title{ 
	Computes Analytical Bounds from Accounting Identity
}
\description{
	Returns analytical bounds from accounting identity on unknown table relationships beta_b, beta_w, from known, observed, table marginals, x, t (and sample size n).
}
\usage{
bounds1(x, t, n)
}
\arguments{
	\item{x}{vector of characteristics, e.g. percentage of blacks in each district}
	\item{t}{vector of characteristics, e.g. percentage of people that voted in each district}
	\item{n}{size of each observation, e.g. number of voters in each district}
}
 
\references{
Gary King (1997). A Solution to the Ecological Inference Problem.  Princeton: Princeton University Press.
}
\author{
  Gary King <<email: king@harvard.edu>> and Molly Roberts <<email: molly.e.roberts@gmail.com>>
}

\examples{
	data(census1910)
	output<-bounds1(x=census1910$x, t=census1910$t, n=census1910$n)
}

     \name{sample}
     \docType{data}
     \alias{sample}
     \title{Sample Data for Black Votes}
     \description{
     	A description for this dataset
     }
     \usage{sample}
     \format{A vector containing 141 observations.}
     \source{Source}
     \references{
     Gary King (1997). \emph{A Solution to the Ecological Inference Problem.}  Princeton: Princeton University Press.
     }
     \keyword{datasets}

\name{ei.sim}
\alias{ei.sim}

\title{ 
	Simulate EI Solution via Importance Sampling
}
\description{
	Simulate EI solution via importance sampling
}
\usage{
ei.sim(ei.object)
}
\arguments{
	\item{ei.object}{\code{ei} object}
}
 
\references{
Gary King (1997). A Solution to the Ecological Inference Problem.  Princeton: Princeton University Press.
}
\author{
  Gary King <<email: king@harvard.edu>> and Molly Roberts <<email: molly.e.roberts@gmail.com>>
}

\name{plot.ei}
\alias{plot.ei}

\title{
Plotting Ecological Inference Estimates
}
\description{
`plot' method for the class `ei'.
}
\usage{
\method{plot}{ei}(x, \dots)
}

\arguments{
  \item{x}{An \code{ei} object from the function \code{ei}.}
  \item{\dots}{A list of options to return in graphs.  See values below.} 
}
 
\details{
	Returns any of a set of possible graphical objects, mirroring those in the examples in King (1997).
	  Graphical option \code{lci} is a logical value specifying the use of the Law of Conservation of Ink, where the implicit information in the data is represented through color gradients, i.e. the color of the line is a function of the length of the tomography line.  This can be passed as an argument and is used for ``tomogD'' and ``tomog'' plots.
}

\value{
	\item{tomogD}{Tomography plot with the data only.  See Figure 5.1, page 81.}
	\item{tomog}{Tomography plot with ML contours.  See Figure 10.2, page 204.}
	\item{tomogCI}{Tomography plot with \eqn{80\%} confidence intervals.  Confidence intervals appear on the screen in red with the remainder of the tomography line in yellow.  The confidence interval portion is also printed thicker than the rest of the line.  See Figure 9.5, page 179.}
	\item{tomogCI95}{Tomography plot with \eqn{95\%} confidence intervals.  Confidence intervals appear on the screen in red with the remainder of the tomography line in yellow.  The confidence interval portion is also printed thicker than the rest of the line.  See Figure 9.5, page 179.}
	\item{tomogE}{Tomography plot with estimated mean posterior \eqn{\beta_i^b} and \eqn{\beta_i^w} points.}
	\item{tomogP}{Tomography plot with mean posterior contours.}
	\item{betab}{Density estimate (i.e., a smooth version of a histogram) of point estimates of \eqn{\beta_i^b}'s with whiskers.} 
	\item{betaw}{Density estimate (i.e., a smooth version of a histogram) of point estimates of \eqn{\beta_i^w}'s with whiskers.} 
	\item{xt}{Basic \eqn{X_i} by \eqn{T_i} scatterplot.}
	\item{xtc}{Basic \eqn{X_i} by \eqn{T_i} scatterplot with circles sized proportional to \eqn{N_i}.}
	\item{xtfit}{\eqn{X_i} by \eqn{T_i} plot with estimated \eqn{E(T_i|X_i)} and conditional \eqn{80\%} confidence intervals.  See Figure 10.3, page 206.}
	\item{xtfitg}{\code{xtfit} with Goodman's regression line superimposed.}
	\item{estsims}{All the simulated \eqn{\beta_i^b}'s by all the simulated \eqn{\beta_i^w}'s.  The simulations should take roughly the same shape of the mean posterior contours, except for those sampled from outlier tomography lines.}
	\item{boundXb}{\eqn{X_i} by the bounds on \eqn{\beta_i^b} (each precinct appears as one vertical line), see the lines in the left graph in Figure 13.2, page 238.}
	\item{boundXw}{\eqn{X_i} by the bounds on \eqn{\beta_i^w} (each precinct appears as one vertical line), see the lines in the right graph in Figure 13.2, page 238.}
	\item{truth}{Compares truth to estimates at the district and precinct-level.  Requires \code{truth} in the \code{ei} object.  See Figures 10.4 (page 208) and 10.5 (page 210).}
    \item{movieD}{For each observation, one tomography plot appears with the line for the particular observation darkened.  After the graph for each observation appears, the user can choose to view the next observation (hit return), jump to a specific observation number (type in the number and hit return), or stop (hit "s" and return).}
  \item{movie}{For each observation, one page of graphics appears with
    the posterior distribution of \eqn{\beta_i^b} and \eqn{\beta_i^w}
    and a plot of the simulated values of  \eqn{\beta_i^b} and
    \eqn{\beta_i^w} from the tomography line.  The user can choose to
    view the next observation (hit return), jump to a specific
    observation number (type in the number and hit return), or stop (hit
    ``s" and return).}
    }
\references{
Gary King (1997). A Solution to the Ecological Inference Problem.  Princeton: Princeton University Press.
}
\author{
  Gary King <<email: king@harvard.edu>> and Molly Roberts <<email: molly.e.roberts@gmail.com>>
}
\examples{
data(sample)
formula = t ~ x
dbuf <- ei(formula=formula, total="n",data=sample)
plot(dbuf, "tomog")
plot(dbuf, "tomog", "betab", "betaw", "xtfit")
}

\name{plot.ei}
\alias{plot.ei}

\title{
Plotting Ecological Inference Estimates
}
\description{
`plot' method for the class `ei'.
}
\usage{
plot.ei(ei.object, ...)
}

\arguments{
  \item{ei.object}{An \code{ei} object from the function \code{ei}.}
\item{\dots}{A list of options to return in graphs.  See values below.} 
}
 
\details{
%%  ~~ If necessary, more details than the description above ~~
}

\value{
	\item{tomogD}{Tomography plot with the data only.  See Figure 5.1, page 81.}
	\item{tomog}{Tomography plot with ML contours.  See Figure 10.2, page 204.}
	\item{tomogCI}{Tomography plot with \eqn{80\%} confidence intervals.  Confidence intervals appear on the screen in red with the remainder of the tomography line in yellow.  The confidence interval portion is also printed thicker than the rest of the line.  See Figure 9.5, page 179.}
	\item{tomogCI95}{Tomography plot with \eqn{95\%} confidence intervals.  Confidence intervals appear on the screen in red with the remainder of the tomography line in yellow.  The confidence interval portion is also printed thicker than the rest of the line.  See Figure 9.5, page 179.}
	\item{tomogE}{Tomography plot with estimated mean posterior \eqn{\beta_i^b} and \eqn{\beta_i^w} points.}
	\item{tomogP}{Tomography plot with mean posterior contours.}
	\item{betab}{Density estimate (i.e., a smooth version of a histogram) of point estimates of \eqn{\beta_i^b}'s with whiskers.} 
	\item{betaw}{Density estimate (i.e., a smooth version of a histogram) of point estimates of \eqn{\beta_i^w}'s with whiskers.} 
	\item{xt}{Basic \eqn{X_i} by \eqn{T_i} scatterplot.}
	\item{xtc}{Basic \eqn{X_i} by \eqn{T_i} scatterplot with circles sized proportional to \eqn{N_i}.}
	\item{xtfit}{\eqn{X_i} by \eqn{T_i} plot with estimated \eqn{E(T_i|X_i)} and conditional \eqn{80\%} confidence intervals.  See Figure 10.3, page 206.}
	\item{xtfitg}{\code{xtfit} with Goodman's regression line superimposed.}
	\item{estsims}{All the simulated \eqn{\beta_i^b}'s by all the simulated \eqn{\beta_i^w}'s.  The simulations should take roughly the same shape of the mean posterior contours, except for those sampled from outlier tomography lines.}
	\item{boundXb}{\eqn{X_i} by the bounds on \eqn{\beta_i^b} (each precinct appears as one vertical line), see the lines in the left graph in Figure 13.2, page 238.}
	\item{boundXw}{\eqn{X_i} by the bounds on \eqn{\beta_i^w} (each precinct appears as one vertical line), see the lines in the right graph in Figure 13.2, page 238.}
	\item{truth}{Compares truth to estimates at the district and precinct-level.  Requires \code{truth} in the \code{ei} object.  See Figures 10.4 (page 208) and 10.5 (page 210).}

}
\references{
Gary King (1997). A Solution to the Ecological Inference Problem.  Princeton: Princeton University Press.
}
\author{
%%  ~~who you are~~
}
\examples{
data(sample)
form <- t ~ x
dbuf <- ei(form,total="n",data=sample)
summary(dbuf)
plot(dbufsim, "tomog", "betab", "betaw", "xtfit")
}

     \name{nj}
     \docType{data}
     \alias{nj}
     \title{Nonminority Turnout in New Jersey}
     \description{
     	A description for this dataset
     }
     \usage{nj}
     \format{A data frame containing 493 observations.}
     \source{Gary King, 1997, "Replication data for: A Solution to the Ecological Inference Problem: Reconstructing Individual Behavior from Aggregate Data", http://hdl.handle.net/1902.1/LWMMKUTYXS UNF:3:DRWozWd89+vNLO7lY2AHbg== IQSS Dataverse Network [Distributor] V3 [Version]}
     \references{
     Gary King (1997). \emph{A Solution to the Ecological Inference Problem.}  Princeton: Princeton University Press.  Section 1.4:24-5.
     }
     \keyword{datasets}

     \name{matproii}
     \docType{data}
     \alias{matproii}
     \title{Voter Registration by Race in Southern States}
     \description{
     	Aggregate voter registration and fraction black, in counties in Florida, Louisiana, North Carolina and South Carolina
     }
     \usage{matproii}
     \format{A data frame containing 268 observations.}
     \source{Gary King, 1997, "Replication data for: A Solution to the Ecological Inference Problem: Reconstructing Individual Behavior from Aggregate Data", http://hdl.handle.net/1902.1/LWMMKUTYXS UNF:3:DRWozWd89+vNLO7lY2AHbg== IQSS Dataverse Network [Distributor] V3 [Version]}
     \references{
     Gary King (1997). \emph{A Solution to the Ecological Inference Problem.}  Princeton: Princeton University Press.  Chapter 10.
     }
     \keyword{datasets}

\name{tomogRxC3d}
\alias{tomogRxC3d}

\title{
Plotting 2x3 Ecological Inference Estimates in 3 dimensions
}
\description{
A tomography plot in 3 dimensions for RxC Ecological Inference data and an estimated Ecological Inference model in RxC data.}
\usage{
tomogRxC3d(formula, data, total=NULL, lci=TRUE, estimates=FALSE, ci=FALSE, level=.95, 
	seed=1234, color=hcl(h=30,c=100,l=60), transparency=.75, light=FALSE, rotate=TRUE)
}

\arguments{
  \item{formula}{A formula of the form \code{cbind(col1, col2,...)~cbind(row1,row2,...)}}
 \item{data}{data that contains the data that corresponds to the
formula}
\item{total}{`total' is the name of the variable in the dataset that
	  contains the number of individuals in each unit}
\item{lci}{logical value specifying the use of the Law of Conservation of Ink, where the implicit information in the data is represented through color gradients, i.e. the color of the plane is a function of the area of the tomography plane.}
\item{estimates}{logical value specifying whether the point estimates of \eqn{\beta}'s are included for each observation on the tomography plot.}
\item{ci}{logical value specifying whether the estimated confidence ellipse is included on the tomography plot.}
\item{level}{numeric value from 0 to 1 specifying the significance level of the confidence ellipse; eg. .95 refers to 95\% confidence ellipse.}
\item{seed}{seed value for model estimation.}
\item{color}{color of tomography planes if lci=F.}
\item{transparency}{numeric value from 0 to 1 specifying transparency of tomography planes; 0 is entirely transparent.}
\item{light}{logical value specifying whether lights should be included in the rgl interface. The inclusion of lights will create shadows in the plot that may distort colors.}
\item{rotate}{logical value specifying whether the plot will rotate for 20 seconds.}
}

\details{
Requires rgl package and rgl viewer.
}

\references{
Gary King (1997). A Solution to the Ecological Inference Problem.  Princeton: Princeton University Press.
}

\author{
  Gary King <<email: king@harvard.edu>>; Molly Roberts <<email: molly.e.roberts@gmail.com>>; Soledad Prillaman <<email: soledadartiz@fas.harvard.edu..
}

\examples{
data(RxCdata)
formula <- cbind(turnout, noturnout) ~ cbind(white, black, hisp)
tomogRxC3d(formula, RxCdata, total=NULL, lci=TRUE, estimates=TRUE, ci=TRUE, transparency=.5, 
	light=FALSE, rotate=FALSE)
}
